
#ifndef __HSCAL_CB_CGEMM_HH
#define __HSCAL_CB_CGEMM_HH



#include "../../../../include/hscalapack.h"

int hscal_cgemm(
    char   *transa, 
    char   *transb, 
    int    *m, 
    int    *n, 
    int    *k, 
    char   *calpha, 
    int    *lda,
    int    *ldb, 
    char   *cbeta, 
    int    *ldc,
    double *tcomp
)
{
    int a_dim1, a_offset, b_dim1, b_offset, c_dim1, c_offset, i__1, i__2, 
	    i__3, i__4, i__5, i__6;
 
    hscalcomplex *alpha = (hscalcomplex*)calpha;
    hscalcomplex *beta  = (hscalcomplex*)cbeta;

    int info;
    hscallogical nota, notb;
    int i__, j, l;
    hscallogical conja, conjb;
    int ncola;
    int nrowa, nrowb;

    nota = hscal_lsame(transa, "N");
    notb = hscal_lsame(transb, "N");
    conja = hscal_lsame(transa, "C");
    conjb = hscal_lsame(transb, "C");
    if (nota) {
	nrowa = *m;
	ncola = *k;
    } else {
	nrowa = *k;
	ncola = *m;
    }
    if (notb) {
	nrowb = *k;
    } else {
	nrowb = *n;
    }

    info   = 0;
    *tcomp = 0;

    if (! nota && ! conja && ! hscal_lsame(transa, "T")) {
	info = 1;
    } else if (! notb && ! conjb && ! hscal_lsame(transb, "T")) {
	info = 2;
    } else if (*m < 0) {
	info = 3;
    } else if (*n < 0) {
	info = 4;
    } else if (*k < 0) {
	info = 5;
    } else if (*lda < max(1,nrowa)) {
	info = 8;
    } else if (*ldb < max(1,nrowb)) {
	info = 10;
    } else if (*ldc < max(1,*m)) {
	info = 13;
    }
    if (info != 0) {
	return HSCAL_SUCCESS;
    }

    if (*m == 0 || *n == 0 || (alpha->r == 0.f && alpha->i == 0.f || *k == 0) 
	    && (beta->r == 1.f && beta->i == 0.f)) {
	return HSCAL_SUCCESS;
    }

    if (alpha->r == 0.f && alpha->i == 0.f) {
	if (beta->r == 0.f && beta->i == 0.f) {
            *tcomp += (*n)*(*m)*4;
	} else {
            *tcomp += (*n)*(*m)*12;
	}
	return HSCAL_SUCCESS;
    }

    if (notb) {
	if (nota) {
	    i__1 = *n;
	    for (j = 1; j <= i__1; ++j) {
		if (beta->r == 0.f && beta->i == 0.f) {
                    *tcomp += (*m)*4;
		} else if (beta->r != 1.f || beta->i != 0.f) {
                    *tcomp += (*m)*12;
		}
                *tcomp += (*k) * (12 + (*m) * 16);
	    }
	} else if (conja) {
	    i__1 = *n;
	    for (j = 1; j <= i__1; ++j) {
		i__2 = *m;
		for (i__ = 1; i__ <= i__2; ++i__) {
                    *tcomp += (*k) * (17) + 2;
		    if (beta->r == 0.f && beta->i == 0.f) {
                       *tcomp += 10;
		    } else {
                       *tcomp += 20;
		    }
		}
	    }
	} else {
	    i__1 = *n;
	    for (j = 1; j <= i__1; ++j) {
		i__2 = *m;
		for (i__ = 1; i__ <= i__2; ++i__) {
                    *tcomp += (*k) * (14) + 2;
		    if (beta->r == 0.f && beta->i == 0.f) {
                       *tcomp += 10;
		    } else {
                       *tcomp += 20;
		    }
		}
	    }
	}
    } else if (nota) {
	if (conjb) {
	    i__1 = *n;
	    for (j = 1; j <= i__1; ++j) {
		if (beta->r == 0.f && beta->i == 0.f) {
                    *tcomp += (*m)*4;
		} else if (beta->r != 1.f || beta->i != 0.f) {
                    *tcomp += (*m)*12;
		}
                *tcomp += (*k)*(15 + (*m) * 16);
	    }
	} else {
	    i__1 = *n;
	    for (j = 1; j <= i__1; ++j) {
		if (beta->r == 0.f && beta->i == 0.f) {
                    *tcomp += (*m)*4;
		} else if (beta->r != 1.f || beta->i != 0.f) {
                    *tcomp += (*m)*12;
		}
                *tcomp += (*k)*(12 + (*m) * 16);
	    }
	}
    } else if (conja) {
	if (conjb) {
	    i__1 = *n;
	    for (j = 1; j <= i__1; ++j) {
		i__2 = *m;
		for (i__ = 1; i__ <= i__2; ++i__) {
                    *tcomp += (*k) * (20) + 2;
		    if (beta->r == 0.f && beta->i == 0.f) {
                        *tcomp += 10;
		    } else {
                        *tcomp += 20;
		    }
		}
	    }
	} else {
	    i__1 = *n;
	    for (j = 1; j <= i__1; ++j) {
		i__2 = *m;
		for (i__ = 1; i__ <= i__2; ++i__) {
                    *tcomp += (*k) * (17) + 2;
		    if (beta->r == 0.f && beta->i == 0.f) {
                        *tcomp += 10;
		    } else {
                        *tcomp += 20;
		    }
		}
	    }
	}
    } else {
	if (conjb) {
	    i__1 = *n;
	    for (j = 1; j <= i__1; ++j) {
		i__2 = *m;
		for (i__ = 1; i__ <= i__2; ++i__) {
                    *tcomp += (*k) * (17) + 2;
		    if (beta->r == 0.f && beta->i == 0.f) {
                        *tcomp += 10;
		    } else {
                        *tcomp += 20;
		    }
		}
	    }
	} else {
	    i__1 = *n;
	    for (j = 1; j <= i__1; ++j) {
		i__2 = *m;
		for (i__ = 1; i__ <= i__2; ++i__) {
                    *tcomp += (*k) * (14) + 2;
		    if (beta->r == 0.f && beta->i == 0.f) {
                        *tcomp += 10;
		    } else {
                        *tcomp += 20;
		    }
		}
	    }
	}
    }

    return HSCAL_SUCCESS;
}

#endif
