
#ifndef __HSCAL_CB_CHERK_HH
#define __HSCAL_CB_CHERK_HH



#include "../../../../include/hscalapack.h"

int hscal_cherk(
    char   *uplo, 
    char   *trans, 
    int    *n, 
    int    *k, 
    char   *calpha, 
    int    *lda,
    char   *cbeta,
    int    *ldc, 
    double *tcomp
)
{
    int a_dim1, a_offset, c_dim1, c_offset, i__1, i__2, i__3, i__4, i__5, 
	    i__6;
    int info;
    int i__, j, l;
    int nrowa;
    hscallogical upper;

    hscalreal *alpha = (hscalreal*)calpha;
    hscalreal *beta  = (hscalreal*)cbeta;

    *tcomp = 0;

    if (hscal_lsame(trans, "N")) {
	nrowa = *n;
    } else {
	nrowa = *k;
    }
    upper = hscal_lsame(uplo, "U");
    info = 0;
    if (! upper && ! hscal_lsame(uplo, "L")) {
	info = 1;
    } else if (! hscal_lsame(trans, "N") && ! hscal_lsame(trans, 
	    "C")) {
	info = 2;
    } else if (*n < 0) {
	info = 3;
    } else if (*k < 0) {
	info = 4;
    } else if (*lda < max(1,nrowa)) {
	info = 7;
    } else if (*ldc < max(1,*n)) {
	info = 10;
    }
    if (info != 0) {
	return HSCAL_SUCCESS;
    }

    if (*n == 0 || (*alpha == 0.f || *k == 0) && *beta == 1.f) {
	return HSCAL_SUCCESS;
    }

    if (*alpha == 0.f) {
	if (upper) {
	    if (*beta == 0.f) {
		i__1 = *n;
		for (j = 1; j <= i__1; ++j) {
                    *tcomp += j*4;
		}
	    } else {
		i__1 = *n;
		for (j = 1; j <= i__1; ++j) {
                    *tcomp += (j-1)*8 + 7;
		}
	    }
	} else {
	    if (*beta == 0.f) {
		i__1 = *n;
		for (j = 1; j <= i__1; ++j) {
		    i__2 = *n;
		    for (i__ = j; i__ <= i__2; ++i__) {
                        *tcomp += 4;
		    }
		}
	    } else {
		i__1 = *n;
		for (j = 1; j <= i__1; ++j) {
                    *tcomp += 7;
		    i__2 = *n;
		    for (i__ = j + 1; i__ <= i__2; ++i__) {
                        *tcomp += 8;
		    }
		}
	    }
	}
	return HSCAL_SUCCESS;
    }

    if (hscal_lsame(trans, "N")) {
	if (upper) {
	    i__1 = *n;
	    for (j = 1; j <= i__1; ++j) {
		if (*beta == 0.f) {
                    *tcomp += j*4;
		} else if (*beta != 1.f) {
                    *tcomp += (j-1)*8 + 7;
		} else {
                    *tcomp += 7;
		}
                *tcomp += (*k)*(24 + (j-1) * 16);
	    }
	} else {
	    i__1 = *n;
	    for (j = 1; j <= i__1; ++j) {
		if (*beta == 0.f) {
		    i__2 = *n;
		    for (i__ = j; i__ <= i__2; ++i__) {
                        *tcomp += 4;
		    }
		} else if (*beta != 1.f) {
                    *tcomp += 7;
		    i__2 = *n;
		    for (i__ = j + 1; i__ <= i__2; ++i__) {
                        *tcomp += 8;
		    }
		} else {
                    *tcomp += 7;
		}
		i__2 = *k;
		for (l = 1; l <= i__2; ++l) {
                    {
                        *tcomp += 24;
			i__3 = *n;
			for (i__ = j + 1; i__ <= i__3; ++i__) {
                            *tcomp += 16;
			}
		    }
		}
	    }
	}
    } else {
	if (upper) {
	    i__1 = *n;
	    for (j = 1; j <= i__1; ++j) {
		i__2 = j - 1;
		for (i__ = 1; i__ <= i__2; ++i__) {
                    *tcomp += (*k)*17;
		    if (*beta == 0.f) {
                        *tcomp += 6;
		    } else {
                        *tcomp += 12;
		    }
		}
                *tcomp += (*k)*16;
		if (*beta == 0.f) {
                    *tcomp += 5;
		} else {
                    *tcomp += 9;
		}
	    }
	} else {
	    i__1 = *n;
	    for (j = 1; j <= i__1; ++j) {
                *tcomp += (*k)*16;
		if (*beta == 0.f) {
                    *tcomp += 5;
		} else {
                    *tcomp += 9;
		}
		i__2 = *n;
		for (i__ = j + 1; i__ <= i__2; ++i__) {
                    *tcomp += (*k)*17;
		    if (*beta == 0.f) {
                        *tcomp += 6;
		    } else {
                        *tcomp += 12;
		    }
		}
	    }
	}
    }

    return HSCAL_SUCCESS;
}

#endif
