
#ifndef __HSCAL_CB_ZGEMV_HH
#define __HSCAL_CB_ZGEMV_HH



#include "../../../../include/hscalapack.h"

int hscal_zgemv(
    char *trans, 
    int *m, 
    int *n, 
    char *calpha, 
    int *lda, 
    int *incx, 
    char *cbeta, 
    int *incy,
    double *tcomp
)
{
    int info;
    int lenx, leny;
    hscallogical noconj;
    hscaldoublecomplex *alpha = (hscaldoublecomplex*)calpha;
    hscaldoublecomplex *beta = (hscaldoublecomplex*)cbeta;

    *tcomp = 0;

    info = 0;

    if (! hscal_lsame(trans, "N") && ! hscal_lsame(trans, "T") && ! hscal_lsame(trans, "C")
	    ) {
	info = 1;
    } else if (*m < 0) {
	info = 2;
    } else if (*n < 0) {
	info = 3;
    } else if (*lda < max(1,*m)) {
	info = 6;
    } else if (*incx == 0) {
	info = 8;
    } else if (*incy == 0) {
	info = 11;
    }

    if (info != 0) {
	return HSCAL_SUCCESS;
    }

/*     Quick return if possible. */
    if (*m == 0 || *n == 0 || alpha->r == 0. && alpha->i == 0. && (beta->r == 
	    1. && beta->i == 0.)) {
	return HSCAL_SUCCESS;
    }

    noconj = hscal_lsame(trans, "T");

/*     Set  LENX  and  LENY, the lengths of the vectors x and y, and set   
       up the start points in  X  and  Y. */
    if (hscal_lsame(trans, "N")) {
	lenx = *n;
	leny = *m;
    } else {
	lenx = *m;
	leny = *n;
    }

/*     Start the operations. In this version the elements of A are   
       accessed sequentially with one pass through A.   
       First form  y := beta*y. */
    if (beta->r != 1. || beta->i != 0.) {
	if (*incy == 1) {
	    if (beta->r == 0. && beta->i == 0.) {
                *tcomp += 3*leny;
	    } else {
                *tcomp += 10*leny;
	    }
	} else {
	    if (beta->r == 0. && beta->i == 0.) {
                *tcomp += 4*leny;
	    } else {
                *tcomp += 11*leny;
	    }
	}
    }

    if (alpha->r == 0. && alpha->i == 0.) {
	return HSCAL_SUCCESS;
    }

    if (hscal_lsame(trans, "N")) {
/*        Form  y := alpha*A*x + y. */
	if (*incy == 1) {
           *tcomp += ((*m) * 13 + 12) * (*n);
	} else {
           *tcomp += ((*m) * 14 + 13) * (*n);
	}
    } else {
/*        Form  y := alpha*A'*x + y  or  y := alpha*conjg( A' )*x + y. */
	if (*incx == 1) {
           if (noconj) {
              *tcomp += ((*m) * 12 + 15) * (*n);
           } else {
              *tcomp += ((*m) * 14 + 15) * (*n);
           }
	} else {
           if (noconj) {
              *tcomp += ((*m) * 13 + 16) * (*n);
           } else {
              *tcomp += ((*m) * 15 + 16) * (*n);
           }
	}
    }

    return HSCAL_SUCCESS;
}

#endif

