
#include <math.h>
#include <sys/time.h>
#include <stdio.h>
#include <stdlib.h>

#include "../../include/hscalapack.h"

/*------------------------------------------------------------------*/

int     hscal_icamax_benchmark(
        int     *n, 
        hscalcomplex *cx, 
        int     *incx
)
{
    return hscal_icamax_benchmark_i(
           (hscalinteger*)n,
           cx,
           (hscalinteger*)incx
    );
}

/*------------------------------------------------------------------*/

int     hscal_icamax_benchmark_i(
        hscalinteger *n, 
        hscalcomplex *cx, 
        hscalinteger *incx
)
{
    /* System generated locals */
    hscalinteger ret_val, i__1, i__2;
    hscalreal r__1, r__2;
    /* Local variables */
    static hscalreal smax;
    static hscalinteger i__, ix;
/*     finds the index of element having max. absolute value.   
       jack dongarra, linpack, 3/11/78.   
       modified 3/93 to return if incx .le. 0.   
       modified 12/3/93, array(1) declarations changed to array(*)   
       Parameter adjustments */
    --cx;
    /* Function Body */
    ret_val = 0;
    if (*n < 1 || *incx <= 0) {
	return ret_val;
    }
    ret_val = 1;
    if (*n == 1) {
	return ret_val;
    }
    if (*incx == 1) {
	goto L20;
    }
/*        code for increment not equal to 1 */
    ix = 1;
    smax = (r__1 = cx[1].r, dabs(r__1)) + (r__2 = hscal_r_imag(&cx[1]), dabs(r__2));
    ix += *incx;
    i__1 = *n;
    for (i__ = 2; i__ <= i__1; ++i__) {
	i__2 = ix;
	if ((r__1 = cx[i__2].r, dabs(r__1)) + (r__2 = hscal_r_imag(&cx[ix]), dabs(
		r__2)) <= smax) {
	    goto L5;
	}
	ret_val = i__;
	i__2 = ix;
	smax = (r__1 = cx[i__2].r, dabs(r__1)) + (r__2 = hscal_r_imag(&cx[ix]), 
		dabs(r__2));
L5:
	ix += *incx;
/* L10: */
    }
    return ret_val;
/*        code for increment equal to 1 */
L20:
    smax = (r__1 = cx[1].r, dabs(r__1)) + (r__2 = hscal_r_imag(&cx[1]), dabs(r__2));
    i__1 = *n;
    for (i__ = 2; i__ <= i__1; ++i__) {
	i__2 = i__;
	if ((r__1 = cx[i__2].r, dabs(r__1)) + (r__2 = hscal_r_imag(&cx[i__]), dabs(
		r__2)) <= smax) {
	    goto L30;
	}
	ret_val = i__;
	i__2 = i__;
	smax = (r__1 = cx[i__2].r, dabs(r__1)) + (r__2 = hscal_r_imag(&cx[i__]), 
		dabs(r__2));
L30:
	;
    }
    return ret_val;
} /* icamax_ */

/*------------------------------------------------------------------*/

int     hscal_idamax_benchmark(
        int     *n, 
        hscaldoublereal *dx, 
        int     *incx
)
{
    return hscal_idamax_benchmark_i(
           (hscalinteger*)n,
           dx,
           (hscalinteger*)incx
    );
}

/*------------------------------------------------------------------*/

int     hscal_idamax_benchmark_i(
        hscalinteger *n, 
        hscaldoublereal *dx, 
        hscalinteger *incx
)
{
    /* System generated locals */
    hscalinteger ret_val, i__1;
    hscaldoublereal d__1;
    /* Local variables */
    static hscaldoublereal dmax__;
    static hscalinteger i__, ix;
/*     finds the index of element having max. absolute value.   
       jack dongarra, linpack, 3/11/78.   
       modified 3/93 to return if incx .le. 0.   
       modified 12/3/93, array(1) declarations changed to array(*)   
       Parameter adjustments */
    --dx;
    /* Function Body */
    ret_val = 0;
    if (*n < 1 || *incx <= 0) {
	return ret_val;
    }
    ret_val = 1;
    if (*n == 1) {
	return ret_val;
    }
    if (*incx == 1) {
	goto L20;
    }
/*        code for increment not equal to 1 */
    ix = 1;
    dmax__ = abs(dx[1]);
    ix += *incx;
    i__1 = *n;
    for (i__ = 2; i__ <= i__1; ++i__) {
	if ((d__1 = dx[ix], abs(d__1)) <= dmax__) {
	    goto L5;
	}
	ret_val = i__;
	dmax__ = (d__1 = dx[ix], abs(d__1));
L5:
	ix += *incx;
/* L10: */
    }
    return ret_val;
/*        code for increment equal to 1 */
L20:
    dmax__ = abs(dx[1]);
    i__1 = *n;
    for (i__ = 2; i__ <= i__1; ++i__) {
	if ((d__1 = dx[i__], abs(d__1)) <= dmax__) {
	    goto L30;
	}
	ret_val = i__;
	dmax__ = (d__1 = dx[i__], abs(d__1));
L30:
	;
    }
    return ret_val;
} /* idamax_ */

/*------------------------------------------------------------------*/

int     hscal_isamax_benchmark(
        int     *n, 
        hscalreal    *sx, 
        int     *incx
)
{
    return hscal_isamax_benchmark_i(
           (hscalinteger*)n,
           sx,
           (hscalinteger*)incx
    );
}

/*------------------------------------------------------------------*/

int     hscal_isamax_benchmark_i(
        hscalinteger *n, 
        hscalreal    *sx, 
        hscalinteger *incx
)
{
    /* System generated locals */
    hscalinteger ret_val, i__1;
    hscalreal r__1;
    /* Local variables */
    static hscalreal smax;
    static hscalinteger i__, ix;
/*     finds the index of element having max. absolute value.   
       jack dongarra, linpack, 3/11/78.   
       modified 3/93 to return if incx .le. 0.   
       modified 12/3/93, array(1) declarations changed to array(*)   
       Parameter adjustments */
    --sx;
    /* Function Body */
    ret_val = 0;
    if (*n < 1 || *incx <= 0) {
	return ret_val;
    }
    ret_val = 1;
    if (*n == 1) {
	return ret_val;
    }
    if (*incx == 1) {
	goto L20;
    }
/*        code for increment not equal to 1 */
    ix = 1;
    smax = dabs(sx[1]);
    ix += *incx;
    i__1 = *n;
    for (i__ = 2; i__ <= i__1; ++i__) {
	if ((r__1 = sx[ix], dabs(r__1)) <= smax) {
	    goto L5;
	}
	ret_val = i__;
	smax = (r__1 = sx[ix], dabs(r__1));
L5:
	ix += *incx;
/* L10: */
    }
    return ret_val;
/*        code for increment equal to 1 */
L20:
    smax = dabs(sx[1]);
    i__1 = *n;
    for (i__ = 2; i__ <= i__1; ++i__) {
	if ((r__1 = sx[i__], dabs(r__1)) <= smax) {
	    goto L30;
	}
	ret_val = i__;
	smax = (r__1 = sx[i__], dabs(r__1));
L30:
	;
    }
    return ret_val;
} /* isamax_ */

/*------------------------------------------------------------------*/

int     hscal_izamax_benchmark(
        int           *n, 
        hscaldoublecomplex *zx, 
        int           *incx
)
{
    return hscal_izamax_benchmark_i(
           (hscalinteger*)n,
           zx,
           (hscalinteger*)incx
    );
}

/*------------------------------------------------------------------*/

int     hscal_izamax_benchmark_i(
        hscalinteger       *n, 
        hscaldoublecomplex *zx, 
        hscalinteger       *incx
)
{
    /* System generated locals */
    hscalinteger ret_val, i__1;
    /* Local variables */
    static hscaldoublereal smax;
    static hscalinteger i__;
    static hscalinteger ix;
/*     finds the index of element having max. absolute value.   
       jack dongarra, 1/15/85.   
       modified 3/93 to return if incx .le. 0.   
       modified 12/3/93, array(1) declarations changed to array(*)   
       Parameter adjustments */
    --zx;
    /* Function Body */
    ret_val = 0;
    if (*n < 1 || *incx <= 0) {
	return ret_val;
    }
    ret_val = 1;
    if (*n == 1) {
	return ret_val;
    }
    if (*incx == 1) {
	goto L20;
    }
/*        code for increment not equal to 1 */
    ix = 1;
    smax = hscal_dcabs1(&zx[1]);
    ix += *incx;
    i__1 = *n;
    for (i__ = 2; i__ <= i__1; ++i__) {
	if (hscal_dcabs1(&zx[ix]) <= smax) {
	    goto L5;
	}
	ret_val = i__;
	smax = hscal_dcabs1(&zx[ix]);
L5:
	ix += *incx;
/* L10: */
    }
    return ret_val;
/*        code for increment equal to 1 */
L20:
    smax = hscal_dcabs1(&zx[1]);
    i__1 = *n;
    for (i__ = 2; i__ <= i__1; ++i__) {
	if (hscal_dcabs1(&zx[i__]) <= smax) {
	    goto L30;
	}
	ret_val = i__;
	smax = hscal_dcabs1(&zx[i__]);
L30:
	;
    }
    return ret_val;
} /* izamax_ */

/*------------------------------------------------------------------*/
