
#include <math.h>
#include <sys/time.h>
#include <stdio.h>
#include <stdlib.h>

#include "../../include/hscalapack.h"

/*------------------------------------------------------------------*/

doublereal hscal_dasum_benchmark_i(
           hscalinteger *n, 
           hscaldoublereal *dx, 
           hscalinteger *incx
)
{
    /* System generated locals */
    hscalinteger i__1, i__2;
    hscaldoublereal ret_val, d__1, d__2, d__3, d__4, d__5, d__6;
    /* Local variables */
    static hscalinteger i__, m;
    static hscaldoublereal dtemp;
    static hscalinteger nincx, mp1;
/*     takes the sum of the absolute values.   
       jack dongarra, linpack, 3/11/78.   
       modified 3/93 to return if incx .le. 0.   
       modified 12/3/93, array(1) declarations changed to array(*)   
       Parameter adjustments */
    --dx;
    /* Function Body */
    ret_val = 0.;
    dtemp = 0.;
    if (*n <= 0 || *incx <= 0) {
	return ret_val;
    }
    if (*incx == 1) {
	goto L20;
    }
/*        code for increment not equal to 1 */
    nincx = *n * *incx;
    i__1 = nincx;
    i__2 = *incx;
    for (i__ = 1; i__2 < 0 ? i__ >= i__1 : i__ <= i__1; i__ += i__2) {
	dtemp += (d__1 = dx[i__], abs(d__1));
/* L10: */
    }
    ret_val = dtemp;
    return ret_val;
/*        code for increment equal to 1   
          clean-up loop */
L20:
    m = *n % 6;
    if (m == 0) {
	goto L40;
    }
    i__2 = m;
    for (i__ = 1; i__ <= i__2; ++i__) {
	dtemp += (d__1 = dx[i__], abs(d__1));
/* L30: */
    }
    if (*n < 6) {
	goto L60;
    }
L40:
    mp1 = m + 1;
    i__2 = *n;
    for (i__ = mp1; i__ <= i__2; i__ += 6) {
	dtemp = dtemp + (d__1 = dx[i__], abs(d__1)) + (d__2 = dx[i__ + 1], 
		abs(d__2)) + (d__3 = dx[i__ + 2], abs(d__3)) + (d__4 = dx[i__ 
		+ 3], abs(d__4)) + (d__5 = dx[i__ + 4], abs(d__5)) + (d__6 = 
		dx[i__ + 5], abs(d__6));
/* L50: */
    }
L60:
    ret_val = dtemp;
    return ret_val;
} /* dasum_ */

/*------------------------------------------------------------------*/

doublereal hscal_dasum_benchmark(
           int     *n, 
           hscaldoublereal *dx, 
           int     *incx
)
{
    return hscal_dasum_benchmark_i(
           (hscalinteger*)n,
           dx,
           (hscalinteger*)incx
    );
}

/*------------------------------------------------------------------*/

doublereal hscal_dzasum_benchmark_i(
           hscalinteger *n, 
           hscaldoublecomplex *zx, 
           hscalinteger *incx
)
{
    /* System generated locals */
    hscalinteger i__1;
    hscaldoublereal ret_val;
    /* Local variables */
    static hscalinteger i__;
    static hscaldoublereal stemp;
    static hscalinteger ix;
/*     takes the sum of the absolute values.   
       jack dongarra, 3/11/78.   
       modified 3/93 to return if incx .le. 0.   
       modified 12/3/93, array(1) declarations changed to array(*)   
       Parameter adjustments */
    --zx;
    /* Function Body */
    ret_val = 0.;
    stemp = 0.;
    if (*n <= 0 || *incx <= 0) {
	return ret_val;
    }
    if (*incx == 1) {
	goto L20;
    }
/*        code for increment not equal to 1 */
    ix = 1;
    i__1 = *n;
    for (i__ = 1; i__ <= i__1; ++i__) {
	stemp += hscal_dcabs1(&zx[ix]);
	ix += *incx;
/* L10: */
    }
    ret_val = stemp;
    return ret_val;
/*        code for increment equal to 1 */
L20:
    i__1 = *n;
    for (i__ = 1; i__ <= i__1; ++i__) {
	stemp += hscal_dcabs1(&zx[i__]);
/* L30: */
    }
    ret_val = stemp;
    return ret_val;
} /* dzasum_ */

/*------------------------------------------------------------------*/

doublereal hscal_dzasum_benchmark(
           int     *n, 
           hscaldoublecomplex *zx, 
           int     *incx
)
{
    return hscal_dzasum_benchmark_i(
           (hscalinteger*)n,
           zx,
           (hscalinteger*)incx
    );
}

/*------------------------------------------------------------------*/

doublereal hscal_sasum_benchmark_i(
           hscalinteger *n, 
           hscalreal    *sx, 
           hscalinteger *incx
)
{
    /* System generated locals */
    hscalinteger i__1, i__2;
    hscalreal ret_val, r__1, r__2, r__3, r__4, r__5, r__6;
    /* Local variables */
    static hscalinteger i__, m, nincx;
    static hscalreal stemp;
    static hscalinteger mp1;
/*     takes the sum of the absolute values.   
       uses unrolled loops for increment equal to one.   
       jack dongarra, linpack, 3/11/78.   
       modified 3/93 to return if incx .le. 0.   
       modified 12/3/93, array(1) declarations changed to array(*)   
       Parameter adjustments */
    --sx;
    /* Function Body */
    ret_val = 0.f;
    stemp = 0.f;
    if (*n <= 0 || *incx <= 0) {
	return ret_val;
    }
    if (*incx == 1) {
	goto L20;
    }
/*        code for increment not equal to 1 */
    nincx = *n * *incx;
    i__1 = nincx;
    i__2 = *incx;
    for (i__ = 1; i__2 < 0 ? i__ >= i__1 : i__ <= i__1; i__ += i__2) {
	stemp += (r__1 = sx[i__], dabs(r__1));
/* L10: */
    }
    ret_val = stemp;
    return ret_val;
/*        code for increment equal to 1   
          clean-up loop */
L20:
    m = *n % 6;
    if (m == 0) {
	goto L40;
    }
    i__2 = m;
    for (i__ = 1; i__ <= i__2; ++i__) {
	stemp += (r__1 = sx[i__], dabs(r__1));
/* L30: */
    }
    if (*n < 6) {
	goto L60;
    }
L40:
    mp1 = m + 1;
    i__2 = *n;
    for (i__ = mp1; i__ <= i__2; i__ += 6) {
	stemp = stemp + (r__1 = sx[i__], dabs(r__1)) + (r__2 = sx[i__ + 1], 
		dabs(r__2)) + (r__3 = sx[i__ + 2], dabs(r__3)) + (r__4 = sx[
		i__ + 3], dabs(r__4)) + (r__5 = sx[i__ + 4], dabs(r__5)) + (
		r__6 = sx[i__ + 5], dabs(r__6));
/* L50: */
    }
L60:
    ret_val = stemp;
    return ret_val;
} /* sasum_ */

/*------------------------------------------------------------------*/

doublereal hscal_sasum_benchmark(
           int     *n, 
           hscalreal    *sx, 
           int     *incx
)
{
    return hscal_sasum_benchmark_i(
           (hscalinteger*)n,
           sx,
           (hscalinteger*)incx
    );
}

/*------------------------------------------------------------------*/

doublereal hscal_scasum_benchmark_i(
           hscalinteger *n, 
           hscalcomplex *cx, 
           hscalinteger *incx
)
{
    /* System generated locals */
    hscalinteger i__1, i__2, i__3;
    hscalreal ret_val, r__1, r__2;

    /* Local variables */
    static hscalinteger i__, nincx;
    static hscalreal stemp;
/*     takes the sum of the absolute values of a hscalcomplex vector and   
       returns a single precision result.   
       jack dongarra, linpack, 3/11/78.   
       modified 3/93 to return if incx .le. 0.   
       modified 12/3/93, array(1) declarations changed to array(*)   
       Parameter adjustments */
    --cx;
    /* Function Body */
    ret_val = 0.f;
    stemp = 0.f;
    if (*n <= 0 || *incx <= 0) {
	return ret_val;
    }
    if (*incx == 1) {
	goto L20;
    }
/*        code for increment not equal to 1 */
    nincx = *n * *incx;
    i__1 = nincx;
    i__2 = *incx;
    for (i__ = 1; i__2 < 0 ? i__ >= i__1 : i__ <= i__1; i__ += i__2) {
	i__3 = i__;
	stemp = stemp + (r__1 = cx[i__3].r, dabs(r__1)) + (r__2 = hscal_r_imag(&cx[
		i__]), dabs(r__2));
/* L10: */
    }
    ret_val = stemp;
    return ret_val;
/*        code for increment equal to 1 */
L20:
    i__2 = *n;
    for (i__ = 1; i__ <= i__2; ++i__) {
	i__1 = i__;
	stemp = stemp + (r__1 = cx[i__1].r, dabs(r__1)) + (r__2 = hscal_r_imag(&cx[
		i__]), dabs(r__2));
/* L30: */
    }
    ret_val = stemp;
    return ret_val;
} /* scasum_ */

/*------------------------------------------------------------------*/

doublereal hscal_scasum_benchmark(
           int     *n, 
           hscalcomplex *cx, 
           int     *incx
)
{
    return hscal_scasum_benchmark_i(
           (hscalinteger*)n,
           cx,
           (hscalinteger*)incx
    );
}

/*------------------------------------------------------------------*/
