
#include <math.h>
#include <sys/time.h>
#include <stdio.h>
#include <stdlib.h>

#include "../../include/hscalapack.h"

/*------------------------------------------------------------------*/

int hscal_caxpy_benchmark(
    int     *n, 
    hscalcomplex *ca, 
    hscalcomplex *cx, 
    int     *incx, 
    hscalcomplex *cy, 
    int     *incy
)
{
    return hscal_caxpy_benchmark_i(
           (hscalinteger*)n,
           ca,
           cx,
           (hscalinteger*)incx,
           cy,
           (hscalinteger*)incy
    );
}

/*------------------------------------------------------------------*/

int hscal_caxpy_benchmark_i(
    hscalinteger *n, 
    hscalcomplex *ca, 
    hscalcomplex *cx, 
    hscalinteger *incx, 
    hscalcomplex *cy, 
    hscalinteger *incy
)
{
    /* System generated locals */
    hscalinteger i__1, i__2, i__3, i__4;
    hscalreal r__1, r__2;
    hscalcomplex q__1, q__2;

    /* Local variables */
    static hscalinteger i__, ix, iy;
/*     constant times a vector plus a vector.   
       jack dongarra, linpack, 3/11/78.   
       modified 12/3/93, array(1) declarations changed to array(*)   
       Parameter adjustments */
    --cy;
    --cx;
    /* Function Body */
    if (*n <= 0) {
	return HSCAL_SUCCESS;
    }
    if ((r__1 = ca->r, dabs(r__1)) + (r__2 = hscal_r_imag(ca), dabs(r__2)) == 0.f) {
	return HSCAL_SUCCESS;
    }
    if (*incx == 1 && *incy == 1) {
	goto L20;
    }
/*        code for unequal increments or equal increments   
            not equal to 1 */
    ix = 1;
    iy = 1;
    if (*incx < 0) {
	ix = (-(*n) + 1) * *incx + 1;
    }
    if (*incy < 0) {
	iy = (-(*n) + 1) * *incy + 1;
    }
    i__1 = *n;
    for (i__ = 1; i__ <= i__1; ++i__) {
	i__2 = iy;
	i__3 = iy;
	i__4 = ix;
	q__2.r = ca->r * cx[i__4].r - ca->i * cx[i__4].i, q__2.i = ca->r * cx[
		i__4].i + ca->i * cx[i__4].r;
	q__1.r = cy[i__3].r + q__2.r, q__1.i = cy[i__3].i + q__2.i;
	cy[i__2].r = q__1.r, cy[i__2].i = q__1.i;
	ix += *incx;
	iy += *incy;
/* L10: */
    }
    return HSCAL_SUCCESS;
/*        code for both increments equal to 1 */
L20:
    i__1 = *n;
    for (i__ = 1; i__ <= i__1; ++i__) {
	i__2 = i__;
	i__3 = i__;
	i__4 = i__;
	q__2.r = ca->r * cx[i__4].r - ca->i * cx[i__4].i, q__2.i = ca->r * cx[
		i__4].i + ca->i * cx[i__4].r;
	q__1.r = cy[i__3].r + q__2.r, q__1.i = cy[i__3].i + q__2.i;
	cy[i__2].r = q__1.r, cy[i__2].i = q__1.i;
/* L30: */
    }
    return HSCAL_SUCCESS;
} /* caxpy_ */

/*------------------------------------------------------------------*/

int hscal_daxpy_benchmark(
    int        *n, 
    hscaldoublereal *da, 
    hscaldoublereal *dx, 
    int        *incx, 
    hscaldoublereal *dy, 
    int        *incy
)
{
    return hscal_daxpy_benchmark_i(
           (hscalinteger*)n,
           da,
           dx,
           (hscalinteger*)incx,
           dy,
           (hscalinteger*)incy
    );
}

/*------------------------------------------------------------------*/

int hscal_daxpy_benchmark_i(
    hscalinteger *n, 
    hscaldoublereal *da, 
    hscaldoublereal *dx, 
    hscalinteger *incx, 
    hscaldoublereal *dy, 
    hscalinteger *incy
)
{
    /* System generated locals */
    hscalinteger i__1;
    /* Local variables */
    static hscalinteger i__, m, ix, iy, mp1;
/*     constant times a vector plus a vector.   
       uses unrolled loops for increments equal to one.   
       jack dongarra, linpack, 3/11/78.   
       modified 12/3/93, array(1) declarations changed to array(*)   
       Parameter adjustments */
    --dy;
    --dx;
    /* Function Body */
    if (*n <= 0) {
	return HSCAL_SUCCESS;
    }
    if (*da == 0.) {
	return HSCAL_SUCCESS;
    }
    if (*incx == 1 && *incy == 1) {
	goto L20;
    }
/*        code for unequal increments or equal increments   
            not equal to 1 */
    ix = 1;
    iy = 1;
    if (*incx < 0) {
	ix = (-(*n) + 1) * *incx + 1;
    }
    if (*incy < 0) {
	iy = (-(*n) + 1) * *incy + 1;
    }
    i__1 = *n;
    for (i__ = 1; i__ <= i__1; ++i__) {
	dy[iy] += *da * dx[ix];
	ix += *incx;
	iy += *incy;
/* L10: */
    }
    return HSCAL_SUCCESS;
/*        code for both increments equal to 1   
          clean-up loop */
L20:
    m = *n % 4;
    if (m == 0) {
	goto L40;
    }
    i__1 = m;
    for (i__ = 1; i__ <= i__1; ++i__) {
	dy[i__] += *da * dx[i__];
/* L30: */
    }
    if (*n < 4) {
	return HSCAL_SUCCESS;
    }
L40:
    mp1 = m + 1;
    i__1 = *n;
    for (i__ = mp1; i__ <= i__1; i__ += 4) {
	dy[i__] += *da * dx[i__];
	dy[i__ + 1] += *da * dx[i__ + 1];
	dy[i__ + 2] += *da * dx[i__ + 2];
	dy[i__ + 3] += *da * dx[i__ + 3];
/* L50: */
    }
    return HSCAL_SUCCESS;
} /* daxpy_ */

/*------------------------------------------------------------------*/

int hscal_saxpy_benchmark(
    int     *n, 
    hscalreal *sa, 
    hscalreal *sx, 
    int     *incx, 
    hscalreal *sy, 
    int     *incy
)
{
    return hscal_saxpy_benchmark_i(
           (hscalinteger*)n,
           sa,
           sx,
           (hscalinteger*)incx,
           sy,
           (hscalinteger*)incy
    );
}

/*------------------------------------------------------------------*/

int hscal_saxpy_benchmark_i(
    hscalinteger *n, 
    hscalreal *sa, 
    hscalreal *sx, 
    hscalinteger *incx, 
    hscalreal *sy, 
    hscalinteger *incy
)
{
    /* System generated locals */
    hscalinteger i__1;
    /* Local variables */
    static hscalinteger i__, m, ix, iy, mp1;
/*     constant times a vector plus a vector.   
       uses unrolled loop for increments equal to one.   
       jack dongarra, linpack, 3/11/78.   
       modified 12/3/93, array(1) declarations changed to array(*)   
       Parameter adjustments */
    --sy;
    --sx;
    /* Function Body */
    if (*n <= 0) {
	return HSCAL_SUCCESS;
    }
    if (*sa == 0.f) {
	return HSCAL_SUCCESS;
    }
    if (*incx == 1 && *incy == 1) {
	goto L20;
    }
/*        code for unequal increments or equal increments   
            not equal to 1 */
    ix = 1;
    iy = 1;
    if (*incx < 0) {
	ix = (-(*n) + 1) * *incx + 1;
    }
    if (*incy < 0) {
	iy = (-(*n) + 1) * *incy + 1;
    }
    i__1 = *n;
    for (i__ = 1; i__ <= i__1; ++i__) {
	sy[iy] += *sa * sx[ix];
	ix += *incx;
	iy += *incy;
/* L10: */
    }
    return HSCAL_SUCCESS;
/*        code for both increments equal to 1   
          clean-up loop */
L20:
    m = *n % 4;
    if (m == 0) {
	goto L40;
    }
    i__1 = m;
    for (i__ = 1; i__ <= i__1; ++i__) {
	sy[i__] += *sa * sx[i__];
/* L30: */
    }
    if (*n < 4) {
	return HSCAL_SUCCESS;
    }
L40:
    mp1 = m + 1;
    i__1 = *n;
    for (i__ = mp1; i__ <= i__1; i__ += 4) {
	sy[i__] += *sa * sx[i__];
	sy[i__ + 1] += *sa * sx[i__ + 1];
	sy[i__ + 2] += *sa * sx[i__ + 2];
	sy[i__ + 3] += *sa * sx[i__ + 3];
/* L50: */
    }
    return HSCAL_SUCCESS;
} /* saxpy_ */

/*------------------------------------------------------------------*/

int hscal_zaxpy_benchmark(  
    int           *n, 
    hscaldoublecomplex *za, 
    hscaldoublecomplex *zx, 
    int           *incx, 
    hscaldoublecomplex *zy, 
    int           *incy
)
{
    return hscal_zaxpy_benchmark_i(
           (hscalinteger*)n,
           za,
           zx,
           (hscalinteger*)incx,
           zy,
           (hscalinteger*)incy
    );
}

/*------------------------------------------------------------------*/

int hscal_zaxpy_benchmark_i(
    hscalinteger *n, 
    hscaldoublecomplex *za, 
    hscaldoublecomplex *zx, 
    hscalinteger *incx, 
    hscaldoublecomplex *zy, 
    hscalinteger *incy
)
{
    /* System generated locals */
    hscalinteger i__1, i__2, i__3, i__4;
    hscaldoublecomplex z__1, z__2;
    /* Local variables */
    static hscalinteger i__;
    static hscalinteger ix, iy;
/*     constant times a vector plus a vector.   
       jack dongarra, 3/11/78.   
       modified 12/3/93, array(1) declarations changed to array(*)   
       Parameter adjustments */
    --zy;
    --zx;
    /* Function Body */
    if (*n <= 0) {
	return HSCAL_SUCCESS;
    }
    if (hscal_dcabs1(za) == 0.) {
	return HSCAL_SUCCESS;
    }
    if (*incx == 1 && *incy == 1) {
	goto L20;
    }
/*        code for unequal increments or equal increments   
            not equal to 1 */
    ix = 1;
    iy = 1;
    if (*incx < 0) {
	ix = (-(*n) + 1) * *incx + 1;
    }
    if (*incy < 0) {
	iy = (-(*n) + 1) * *incy + 1;
    }
    i__1 = *n;
    for (i__ = 1; i__ <= i__1; ++i__) {
	i__2 = iy;
	i__3 = iy;
	i__4 = ix;
	z__2.r = za->r * zx[i__4].r - za->i * zx[i__4].i, z__2.i = za->r * zx[
		i__4].i + za->i * zx[i__4].r;
	z__1.r = zy[i__3].r + z__2.r, z__1.i = zy[i__3].i + z__2.i;
	zy[i__2].r = z__1.r, zy[i__2].i = z__1.i;
	ix += *incx;
	iy += *incy;
/* L10: */
    }
    return HSCAL_SUCCESS;
/*        code for both increments equal to 1 */
L20:
    i__1 = *n;
    for (i__ = 1; i__ <= i__1; ++i__) {
	i__2 = i__;
	i__3 = i__;
	i__4 = i__;
	z__2.r = za->r * zx[i__4].r - za->i * zx[i__4].i, z__2.i = za->r * zx[
		i__4].i + za->i * zx[i__4].r;
	z__1.r = zy[i__3].r + z__2.r, z__1.i = zy[i__3].i + z__2.i;
	zy[i__2].r = z__1.r, zy[i__2].i = z__1.i;
/* L30: */
    }
    return HSCAL_SUCCESS;
} /* zaxpy_ */

/*------------------------------------------------------------------*/
