
#include <math.h>
#include <sys/time.h>
#include <stdio.h>
#include <stdlib.h>

#include "../../include/hscalapack.h"

/*------------------------------------------------------------------*/

int hscal_ccopy_benchmark(
    int     *n, 
    hscalcomplex *cx, 
    int     *incx, 
    hscalcomplex *cy, 
    int     *incy
)
{
    return hscal_ccopy_benchmark_i(
           (hscalinteger*)n,
           cx,
           (hscalinteger*)incx,
           cy,
           (hscalinteger*)incy
    );
}

/*------------------------------------------------------------------*/

int hscal_ccopy_benchmark_i(
    hscalinteger *n, 
    hscalcomplex *cx, 
    hscalinteger *incx, 
    hscalcomplex *cy, 
    hscalinteger *incy
)
{
    /* System generated locals */
    hscalinteger i__1, i__2, i__3;
    /* Local variables */
    static hscalinteger i__, ix, iy;
/*     copies a vector, x, to a vector, y.   
       jack dongarra, linpack, 3/11/78.   
       modified 12/3/93, array(1) declarations changed to array(*)   
       Parameter adjustments */
    --cy;
    --cx;
    /* Function Body */
    if (*n <= 0) {
	return HSCAL_SUCCESS;
    }
    if (*incx == 1 && *incy == 1) {
	goto L20;
    }
/*        code for unequal increments or equal increments   
            not equal to 1 */
    ix = 1;
    iy = 1;
    if (*incx < 0) {
	ix = (-(*n) + 1) * *incx + 1;
    }
    if (*incy < 0) {
	iy = (-(*n) + 1) * *incy + 1;
    }
    i__1 = *n;
    for (i__ = 1; i__ <= i__1; ++i__) {
	i__2 = iy;
	i__3 = ix;
	cy[i__2].r = cx[i__3].r, cy[i__2].i = cx[i__3].i;
	ix += *incx;
	iy += *incy;
/* L10: */
    }
    return HSCAL_SUCCESS;
/*        code for both increments equal to 1 */
L20:
    i__1 = *n;
    for (i__ = 1; i__ <= i__1; ++i__) {
	i__2 = i__;
	i__3 = i__;
	cy[i__2].r = cx[i__3].r, cy[i__2].i = cx[i__3].i;
/* L30: */
    }
    return HSCAL_SUCCESS;
} /* ccopy_ */

/*------------------------------------------------------------------*/

int hscal_dcopy_benchmark(
    int        *n, 
    hscaldoublereal *dx, 
    int        *incx, 
    hscaldoublereal *dy, 
    int        *incy
)
{
    return hscal_dcopy_benchmark_i(
           (hscalinteger*)n,
           dx,
           (hscalinteger*)incx,
           dy,
           (hscalinteger*)incy
    );
}

/*------------------------------------------------------------------*/

int hscal_dcopy_benchmark_i(
    hscalinteger *n, 
    hscaldoublereal *dx, 
    hscalinteger *incx, 
    hscaldoublereal *dy, 
    hscalinteger *incy
)
{
    /* System generated locals */
    hscalinteger i__1;
    /* Local variables */
    static hscalinteger i__, m, ix, iy, mp1;
/*     copies a vector, x, to a vector, y.   
       uses unrolled loops for increments equal to one.   
       jack dongarra, linpack, 3/11/78.   
       modified 12/3/93, array(1) declarations changed to array(*)   
       Parameter adjustments */
    --dy;
    --dx;
    /* Function Body */
    if (*n <= 0) {
	return HSCAL_SUCCESS;
    }
    if (*incx == 1 && *incy == 1) {
	goto L20;
    }
/*        code for unequal increments or equal increments   
            not equal to 1 */
    ix = 1;
    iy = 1;
    if (*incx < 0) {
	ix = (-(*n) + 1) * *incx + 1;
    }
    if (*incy < 0) {
	iy = (-(*n) + 1) * *incy + 1;
    }
    i__1 = *n;
    for (i__ = 1; i__ <= i__1; ++i__) {
	dy[iy] = dx[ix];
	ix += *incx;
	iy += *incy;
/* L10: */
    }
    return HSCAL_SUCCESS;
/*        code for both increments equal to 1   
          clean-up loop */
L20:
    m = *n % 7;
    if (m == 0) {
	goto L40;
    }
    i__1 = m;
    for (i__ = 1; i__ <= i__1; ++i__) {
	dy[i__] = dx[i__];
/* L30: */
    }
    if (*n < 7) {
	return HSCAL_SUCCESS;
    }
L40:
    mp1 = m + 1;
    i__1 = *n;
    for (i__ = mp1; i__ <= i__1; i__ += 7) {
	dy[i__] = dx[i__];
	dy[i__ + 1] = dx[i__ + 1];
	dy[i__ + 2] = dx[i__ + 2];
	dy[i__ + 3] = dx[i__ + 3];
	dy[i__ + 4] = dx[i__ + 4];
	dy[i__ + 5] = dx[i__ + 5];
	dy[i__ + 6] = dx[i__ + 6];
/* L50: */
    }
    return HSCAL_SUCCESS;
} /* dcopy_ */

/*------------------------------------------------------------------*/

int hscal_icopy_benchmark(
    int     *n, 
    int     *cx, 
    int     *incx, 
    int     *cy, 
    int     *incy
)
{
    return hscal_icopy_benchmark_i(
           (hscalinteger*)n,
           cx,
           (hscalinteger*)incx,
           cy,
           (hscalinteger*)incy
    );
}

/*------------------------------------------------------------------*/

int hscal_icopy_benchmark_i(
    hscalinteger *n, 
    int     *dx, 
    hscalinteger *incx, 
    int     *dy, 
    hscalinteger *incy
)
{
    /* System generated locals */
    hscalinteger i__1;
    /* Local variables */
    static hscalinteger i__, m, ix, iy, mp1;
/*     copies a vector, x, to a vector, y.   
       uses unrolled loops for increments equal to one.   
       jack dongarra, linpack, 3/11/78.   
       modified 12/3/93, array(1) declarations changed to array(*)   
       Parameter adjustments */
    --dy;
    --dx;
    /* Function Body */
    if (*n <= 0) {
	return HSCAL_SUCCESS;
    }
    if (*incx == 1 && *incy == 1) {
	goto L20;
    }
/*        code for unequal increments or equal increments   
            not equal to 1 */
    ix = 1;
    iy = 1;
    if (*incx < 0) {
	ix = (-(*n) + 1) * *incx + 1;
    }
    if (*incy < 0) {
	iy = (-(*n) + 1) * *incy + 1;
    }
    i__1 = *n;
    for (i__ = 1; i__ <= i__1; ++i__) {
	dy[iy] = dx[ix];
	ix += *incx;
	iy += *incy;
/* L10: */
    }
    return HSCAL_SUCCESS;
/*        code for both increments equal to 1   
          clean-up loop */
L20:
    m = *n % 7;
    if (m == 0) {
	goto L40;
    }
    i__1 = m;
    for (i__ = 1; i__ <= i__1; ++i__) {
	dy[i__] = dx[i__];
/* L30: */
    }
    if (*n < 7) {
	return HSCAL_SUCCESS;
    }
L40:
    mp1 = m + 1;
    i__1 = *n;
    for (i__ = mp1; i__ <= i__1; i__ += 7) {
	dy[i__] = dx[i__];
	dy[i__ + 1] = dx[i__ + 1];
	dy[i__ + 2] = dx[i__ + 2];
	dy[i__ + 3] = dx[i__ + 3];
	dy[i__ + 4] = dx[i__ + 4];
	dy[i__ + 5] = dx[i__ + 5];
	dy[i__ + 6] = dx[i__ + 6];
/* L50: */
    }
    return HSCAL_SUCCESS;
}

/*------------------------------------------------------------------*/

int hscal_scopy_benchmark(
    int     *n, 
    hscalreal    *sx, 
    int     *incx,
    hscalreal    *sy, 
    int     *incy
)
{
    return hscal_scopy_benchmark_i(
           (hscalinteger*)n,
           sx,
           (hscalinteger*)incx,
           sy,
           (hscalinteger*)incy
    );
}

/*------------------------------------------------------------------*/

int hscal_scopy_benchmark_i(
    hscalinteger *n, 
    hscalreal *sx, 
    hscalinteger *incx,
    hscalreal *sy, 
    hscalinteger *incy
)
{
    /* System generated locals */
    hscalinteger i__1;
    /* Local variables */
    static hscalinteger i__, m, ix, iy, mp1;
/*     copies a vector, x, to a vector, y.   
       uses unrolled loops for increments equal to 1.   
       jack dongarra, linpack, 3/11/78.   
       modified 12/3/93, array(1) declarations changed to array(*)   
       Parameter adjustments */
    --sy;
    --sx;
    /* Function Body */
    if (*n <= 0) {
	return HSCAL_SUCCESS;
    }
    if (*incx == 1 && *incy == 1) {
	goto L20;
    }
/*        code for unequal increments or equal increments   
            not equal to 1 */
    ix = 1;
    iy = 1;
    if (*incx < 0) {
	ix = (-(*n) + 1) * *incx + 1;
    }
    if (*incy < 0) {
	iy = (-(*n) + 1) * *incy + 1;
    }
    i__1 = *n;
    for (i__ = 1; i__ <= i__1; ++i__) {
	sy[iy] = sx[ix];
	ix += *incx;
	iy += *incy;
/* L10: */
    }
    return HSCAL_SUCCESS;
/*        code for both increments equal to 1   
          clean-up loop */
L20:
    m = *n % 7;
    if (m == 0) {
	goto L40;
    }
    i__1 = m;
    for (i__ = 1; i__ <= i__1; ++i__) {
	sy[i__] = sx[i__];
/* L30: */
    }
    if (*n < 7) {
	return HSCAL_SUCCESS;
    }
L40:
    mp1 = m + 1;
    i__1 = *n;
    for (i__ = mp1; i__ <= i__1; i__ += 7) {
	sy[i__] = sx[i__];
	sy[i__ + 1] = sx[i__ + 1];
	sy[i__ + 2] = sx[i__ + 2];
	sy[i__ + 3] = sx[i__ + 3];
	sy[i__ + 4] = sx[i__ + 4];
	sy[i__ + 5] = sx[i__ + 5];
	sy[i__ + 6] = sx[i__ + 6];
/* L50: */
    }
    return HSCAL_SUCCESS;
} /* scopy_ */

/*------------------------------------------------------------------*/

int hscal_zcopy_benchmark(
    int     *n, 
    hscaldoublecomplex *zx, 
    int     *incx, 
    hscaldoublecomplex *zy, 
    int     *incy
)
{
    return hscal_zcopy_benchmark_i(
           (hscalinteger*)n,
           zx,
           (hscalinteger*)incx,
           zy,
           (hscalinteger*)incy
    );
}

/*------------------------------------------------------------------*/

int hscal_zcopy_benchmark_i(
    hscalinteger *n, 
    hscaldoublecomplex *zx, 
    hscalinteger *incx, 
    hscaldoublecomplex *zy, 
    hscalinteger *incy
)
{
    /* System generated locals */
    hscalinteger i__1, i__2, i__3;
    /* Local variables */
    static hscalinteger i__, ix, iy;
/*     copies a vector, x, to a vector, y.   
       jack dongarra, linpack, 4/11/78.   
       modified 12/3/93, array(1) declarations changed to array(*)   
       Parameter adjustments */
    --zy;
    --zx;
    /* Function Body */
    if (*n <= 0) {
	return HSCAL_SUCCESS;
    }
    if (*incx == 1 && *incy == 1) {
	goto L20;
    }
/*        code for unequal increments or equal increments   
            not equal to 1 */
    ix = 1;
    iy = 1;
    if (*incx < 0) {
	ix = (-(*n) + 1) * *incx + 1;
    }
    if (*incy < 0) {
	iy = (-(*n) + 1) * *incy + 1;
    }
    i__1 = *n;
    for (i__ = 1; i__ <= i__1; ++i__) {
	i__2 = iy;
	i__3 = ix;
	zy[i__2].r = zx[i__3].r, zy[i__2].i = zx[i__3].i;
	ix += *incx;
	iy += *incy;
/* L10: */
    }
    return HSCAL_SUCCESS;
/*        code for both increments equal to 1 */
L20:
    i__1 = *n;
    for (i__ = 1; i__ <= i__1; ++i__) {
	i__2 = i__;
	i__3 = i__;
	zy[i__2].r = zx[i__3].r, zy[i__2].i = zx[i__3].i;
/* L30: */
    }
    return HSCAL_SUCCESS;
} /* zcopy_ */

/*------------------------------------------------------------------*/
