
#include <math.h>
#include <sys/time.h>
#include <stdio.h>
#include <stdlib.h>

#include "../../include/hscalapack.h"

/*------------------------------------------------------------------*/

void hscal_cdotc_benchmark_i(
     hscalcomplex * ret_val, 
     hscalinteger *n, 
     hscalcomplex *cx, 
     hscalinteger *incx, 
     hscalcomplex *cy, 
     hscalinteger *incy
)
{
    /* System generated locals */
    hscalinteger i__1, i__2;
    hscalcomplex q__1, q__2, q__3;

    /* Local variables */
    static hscalinteger i__;
    static hscalcomplex ctemp;
    static hscalinteger ix, iy;
/*     forms the dot product of two vectors, conjugating the first   
       vector.   
       jack dongarra, linpack,  3/11/78.   
       modified 12/3/93, array(1) declarations changed to array(*)   
       Parameter adjustments */
    --cy;
    --cx;
    /* Function Body */
    ctemp.r = 0.f, ctemp.i = 0.f;
     ret_val->r = 0.f,  ret_val->i = 0.f;
    if (*n <= 0) {
	return ;
    }
    if (*incx == 1 && *incy == 1) {
	goto L20;
    }
/*        code for unequal increments or equal increments   
            not equal to 1 */
    ix = 1;
    iy = 1;
    if (*incx < 0) {
	ix = (-(*n) + 1) * *incx + 1;
    }
    if (*incy < 0) {
	iy = (-(*n) + 1) * *incy + 1;
    }
    i__1 = *n;
    for (i__ = 1; i__ <= i__1; ++i__) {
	hscal_r_cnjg(&q__3, &cx[ix]);
	i__2 = iy;
	q__2.r = q__3.r * cy[i__2].r - q__3.i * cy[i__2].i, q__2.i = q__3.r * 
		cy[i__2].i + q__3.i * cy[i__2].r;
	q__1.r = ctemp.r + q__2.r, q__1.i = ctemp.i + q__2.i;
	ctemp.r = q__1.r, ctemp.i = q__1.i;
	ix += *incx;
	iy += *incy;
/* L10: */
    }
     ret_val->r = ctemp.r,  ret_val->i = ctemp.i;
    return ;
/*        code for both increments equal to 1 */
L20:
    i__1 = *n;
    for (i__ = 1; i__ <= i__1; ++i__) {
	hscal_r_cnjg(&q__3, &cx[i__]);
	i__2 = i__;
	q__2.r = q__3.r * cy[i__2].r - q__3.i * cy[i__2].i, q__2.i = q__3.r * 
		cy[i__2].i + q__3.i * cy[i__2].r;
	q__1.r = ctemp.r + q__2.r, q__1.i = ctemp.i + q__2.i;
	ctemp.r = q__1.r, ctemp.i = q__1.i;
/* L30: */
    }
     ret_val->r = ctemp.r,  ret_val->i = ctemp.i;
    return ;
} /* cdotc_ */

/*------------------------------------------------------------------*/

void hscal_cdotc_benchmark(
     hscalcomplex *ret_val, 
     int     *n, 
     hscalcomplex *cx, 
     int     *incx, 
     hscalcomplex *cy, 
     int     *incy
)
{
     hscal_cdotc_benchmark_i(
          ret_val,
          (hscalinteger*)n,
          cx,
          (hscalinteger*)incx,
          cy,
          (hscalinteger*)incy
     );
}

/*------------------------------------------------------------------*/

void hscal_zdotc_benchmark_i(
     hscaldoublecomplex * ret_val, 
     hscalinteger       * n, 
     hscaldoublecomplex * zx, 
     hscalinteger       * incx, 
     hscaldoublecomplex * zy, 
     hscalinteger       * incy
)
{
    /* System generated locals */
    hscalinteger i__1, i__2;
    hscaldoublecomplex z__1, z__2, z__3;

    /* Local variables */
    static hscalinteger i__;
    static hscaldoublecomplex ztemp;
    static hscalinteger ix, iy;
/*     forms the dot product of a vector.   
       jack dongarra, 3/11/78.   
       modified 12/3/93, array(1) declarations changed to array(*)   
       Parameter adjustments */
    --zy;
    --zx;
    /* Function Body */
    ztemp.r = 0., ztemp.i = 0.;
     ret_val->r = 0.,  ret_val->i = 0.;
    if (*n <= 0) {
	return ;
    }
    if (*incx == 1 && *incy == 1) {
	goto L20;
    }
/*        code for unequal increments or equal increments   
            not equal to 1 */
    ix = 1;
    iy = 1;
    if (*incx < 0) {
	ix = (-(*n) + 1) * *incx + 1;
    }
    if (*incy < 0) {
	iy = (-(*n) + 1) * *incy + 1;
    }
    i__1 = *n;
    for (i__ = 1; i__ <= i__1; ++i__) {
	hscal_d_cnjg(&z__3, &zx[ix]);
	i__2 = iy;
	z__2.r = z__3.r * zy[i__2].r - z__3.i * zy[i__2].i, z__2.i = z__3.r * 
		zy[i__2].i + z__3.i * zy[i__2].r;
	z__1.r = ztemp.r + z__2.r, z__1.i = ztemp.i + z__2.i;
	ztemp.r = z__1.r, ztemp.i = z__1.i;
	ix += *incx;
	iy += *incy;
/* L10: */
    }
     ret_val->r = ztemp.r,  ret_val->i = ztemp.i;
    return ;
/*        code for both increments equal to 1 */
L20:
    i__1 = *n;
    for (i__ = 1; i__ <= i__1; ++i__) {
	hscal_d_cnjg(&z__3, &zx[i__]);
	i__2 = i__;
	z__2.r = z__3.r * zy[i__2].r - z__3.i * zy[i__2].i, z__2.i = z__3.r * 
		zy[i__2].i + z__3.i * zy[i__2].r;
	z__1.r = ztemp.r + z__2.r, z__1.i = ztemp.i + z__2.i;
	ztemp.r = z__1.r, ztemp.i = z__1.i;
/* L30: */
    }
     ret_val->r = ztemp.r,  ret_val->i = ztemp.i;
    return ;
} /* zdotc_ */

/*------------------------------------------------------------------*/

void hscal_zdotc_benchmark(
     hscaldoublecomplex * ret_val, 
     int           * n, 
     hscaldoublecomplex * zx, 
     int           * incx, 
     hscaldoublecomplex * zy, 
     int           * incy
)
{
     hscal_zdotc_benchmark_i(
          ret_val,
          (hscalinteger*)n,
          zx,
          (hscalinteger*)incx,
          zy,
          (hscalinteger*)incy
     );
}

/*------------------------------------------------------------------*/
