
#include <math.h>
#include <sys/time.h>
#include <stdio.h>
#include <stdlib.h>

#include "../../include/hscalapack.h"

/*------------------------------------------------------------------*/

doublereal hscal_dnrm2_benchmark_i(
           hscalinteger *n, 
           hscaldoublereal *x, 
           hscalinteger *incx
)
{
/*        The following loop is equivalent to this call to the LAPACK   
          auxiliary routine:   
          CALL DLASSQ( N, X, INCX, SCALE, SSQ ) */
    /* System generated locals */
    hscalinteger i__1, i__2;
    hscaldoublereal ret_val, d__1;

    /* Local variables */
    static hscaldoublereal norm, scale, absxi;
    static hscalinteger ix;
    static hscaldoublereal ssq;
/*  DNRM2 returns the euclidean norm of a vector via the function   
    name, so that   
       DNRM2 := sqrt( x'*x )   
    -- This version written on 25-October-1982.   
       Modified on 14-October-1993 to inline the call to DLASSQ.   
       Sven Hammarling, Nag Ltd.   
       Parameter adjustments */
    --x;
    /* Function Body */
    if (*n < 1 || *incx < 1) {
	norm = 0.;
    } else if (*n == 1) {
	norm = abs(x[1]);
    } else {
	scale = 0.;
	ssq = 1.;


	i__1 = (*n - 1) * *incx + 1;
	i__2 = *incx;
	for (ix = 1; i__2 < 0 ? ix >= i__1 : ix <= i__1; ix += i__2) {
	    if (x[ix] != 0.) {
		absxi = (d__1 = x[ix], abs(d__1));
		if (scale < absxi) {
/* Computing 2nd power */
		    d__1 = scale / absxi;
		    ssq = ssq * (d__1 * d__1) + 1.;
		    scale = absxi;
		} else {
/* Computing 2nd power */
		    d__1 = absxi / scale;
		    ssq += d__1 * d__1;
		}
	    }
/* L10: */
	}
	norm = scale * sqrt(ssq);
    }

    ret_val = norm;
    return ret_val;

/*     End of DNRM2. */

} /* dnrm2_ */

/*------------------------------------------------------------------*/

doublereal hscal_dnrm2_benchmark(
           int     *n, 
           hscaldoublereal *x, 
           int     *incx
)
{
    return hscal_dnrm2_benchmark_i(
           (hscalinteger*)n,
           x,
           (hscalinteger*)incx
    );
}

/*------------------------------------------------------------------*/

doublereal hscal_dznrm2_benchmark_i(
           hscalinteger *n, 
           hscaldoublecomplex *x, 
           hscalinteger *incx
)
{
/*        The following loop is equivalent to this call to the LAPACK   
          auxiliary routine:   
          CALL ZLASSQ( N, X, INCX, SCALE, SSQ ) */
    /* System generated locals */
    hscalinteger i__1, i__2, i__3;
    hscaldoublereal ret_val, d__1;

    /* Local variables */
    static hscaldoublereal temp, norm, scale;
    static hscalinteger ix;
    static hscaldoublereal ssq;
/*  DZNRM2 returns the euclidean norm of a vector via the function   
    name, so that   
       DZNRM2 := sqrt( conjg( x' )*x )   
    -- This version written on 25-October-1982.   
       Modified on 14-October-1993 to inline the call to ZLASSQ.   
       Sven Hammarling, Nag Ltd.   
       Parameter adjustments */
    --x;
    /* Function Body */
    if (*n < 1 || *incx < 1) {
	norm = 0.;
    } else {
	scale = 0.;
	ssq = 1.;


	i__1 = (*n - 1) * *incx + 1;
	i__2 = *incx;
	for (ix = 1; i__2 < 0 ? ix >= i__1 : ix <= i__1; ix += i__2) {
	    i__3 = ix;
	    if (x[i__3].r != 0.) {
		i__3 = ix;
		temp = (d__1 = x[i__3].r, abs(d__1));
		if (scale < temp) {
/* Computing 2nd power */
		    d__1 = scale / temp;
		    ssq = ssq * (d__1 * d__1) + 1.;
		    scale = temp;
		} else {
/* Computing 2nd power */
		    d__1 = temp / scale;
		    ssq += d__1 * d__1;
		}
	    }
	    if (hscal_d_imag(&x[ix]) != 0.) {
		temp = (d__1 = hscal_d_imag(&x[ix]), abs(d__1));
		if (scale < temp) {
/* Computing 2nd power */
		    d__1 = scale / temp;
		    ssq = ssq * (d__1 * d__1) + 1.;
		    scale = temp;
		} else {
/* Computing 2nd power */
		    d__1 = temp / scale;
		    ssq += d__1 * d__1;
		}
	    }
/* L10: */
	}
	norm = scale * sqrt(ssq);
    }

    ret_val = norm;
    return ret_val;

/*     End of DZNRM2. */

} /* dznrm2_ */

/*------------------------------------------------------------------*/

doublereal hscal_dznrm2_benchmark(
           int     *n, 
           hscaldoublecomplex *x, 
           int     *incx
)
{
    return hscal_dznrm2_benchmark_i(
           (hscalinteger*)n,
           x,
           (hscalinteger*)incx
    );
}

/*------------------------------------------------------------------*/

doublereal hscal_snrm2_benchmark_i(
           hscalinteger *n, 
           hscalreal *x, 
           hscalinteger *incx
)
{
/*        The following loop is equivalent to this call to the LAPACK   
          auxiliary routine:   
          CALL SLASSQ( N, X, INCX, SCALE, SSQ ) */
    /* System generated locals */
    hscalinteger i__1, i__2;
    hscalreal ret_val, r__1;
    /* Builtin functions */
    double sqrt(doublereal);
    /* Local variables */
    static hscalreal norm, scale, absxi;
    static hscalinteger ix;
    static hscalreal ssq;
/*  SNRM2 returns the euclidean norm of a vector via the function   
    name, so that   
       SNRM2 := sqrt( x'*x )   
    -- This version written on 25-October-1982.   
       Modified on 14-October-1993 to inline the call to SLASSQ.   
       Sven Hammarling, Nag Ltd.   
       Parameter adjustments */
    --x;
    /* Function Body */
    if (*n < 1 || *incx < 1) {
	norm = 0.f;
    } else if (*n == 1) {
	norm = dabs(x[1]);
    } else {
	scale = 0.f;
	ssq = 1.f;


	i__1 = (*n - 1) * *incx + 1;
	i__2 = *incx;
	for (ix = 1; i__2 < 0 ? ix >= i__1 : ix <= i__1; ix += i__2) {
	    if (x[ix] != 0.f) {
		absxi = (r__1 = x[ix], dabs(r__1));
		if (scale < absxi) {
/* Computing 2nd power */
		    r__1 = scale / absxi;
		    ssq = ssq * (r__1 * r__1) + 1.f;
		    scale = absxi;
		} else {
/* Computing 2nd power */
		    r__1 = absxi / scale;
		    ssq += r__1 * r__1;
		}
	    }
/* L10: */
	}
	norm = scale * sqrt(ssq);
    }

    ret_val = norm;
    return ret_val;

/*     End of SNRM2. */

} /* snrm2_ */

/*------------------------------------------------------------------*/

doublereal hscal_snrm2_benchmark(
           int  *n, 
           hscalreal *x, 
           int  *incx
)
{
    return hscal_snrm2_benchmark_i(
           (hscalinteger*)n,
           x,
           (hscalinteger*)incx
    );
}

/*------------------------------------------------------------------*/

doublereal hscal_scnrm2_benchmark_i(
           hscalinteger *n, 
           hscalcomplex *x, 
           hscalinteger *incx
)
{
/*        The following loop is equivalent to this call to the LAPACK   
          auxiliary routine:   
          CALL CLASSQ( N, X, INCX, SCALE, SSQ ) */
    /* System generated locals */
    hscalinteger i__1, i__2, i__3;
    hscalreal ret_val, r__1;
    /* Builtin functions */
    double hscal_r_imag(complex *), sqrt(doublereal);
    /* Local variables */
    static hscalreal temp, norm, scale;
    static hscalinteger ix;
    static hscalreal ssq;
/*  SCNRM2 returns the euclidean norm of a vector via the function   
    name, so that   
       SCNRM2 := sqrt( conjg( x' )*x )   
    -- This version written on 25-October-1982.   
       Modified on 14-October-1993 to inline the call to CLASSQ.   
       Sven Hammarling, Nag Ltd.   
       Parameter adjustments */
    --x;
    /* Function Body */
    if (*n < 1 || *incx < 1) {
	norm = 0.f;
    } else {
	scale = 0.f;
	ssq = 1.f;


	i__1 = (*n - 1) * *incx + 1;
	i__2 = *incx;
	for (ix = 1; i__2 < 0 ? ix >= i__1 : ix <= i__1; ix += i__2) {
	    i__3 = ix;
	    if (x[i__3].r != 0.f) {
		i__3 = ix;
		temp = (r__1 = x[i__3].r, dabs(r__1));
		if (scale < temp) {
/* Computing 2nd power */
		    r__1 = scale / temp;
		    ssq = ssq * (r__1 * r__1) + 1.f;
		    scale = temp;
		} else {
/* Computing 2nd power */
		    r__1 = temp / scale;
		    ssq += r__1 * r__1;
		}
	    }
	    if (hscal_r_imag(&x[ix]) != 0.f) {
		temp = (r__1 = hscal_r_imag(&x[ix]), dabs(r__1));
		if (scale < temp) {
/* Computing 2nd power */
		    r__1 = scale / temp;
		    ssq = ssq * (r__1 * r__1) + 1.f;
		    scale = temp;
		} else {
/* Computing 2nd power */
		    r__1 = temp / scale;
		    ssq += r__1 * r__1;
		}
	    }
/* L10: */
	}
	norm = scale * sqrt(ssq);
    }

    ret_val = norm;
    return ret_val;

/*     End of SCNRM2. */

} /* scnrm2_ */

/*------------------------------------------------------------------*/

doublereal hscal_scnrm2_benchmark(
           int     *n, 
           hscalcomplex *x, 
           int     *incx
)
{
    return hscal_scnrm2_benchmark_i(
           (hscalinteger*)n,
           x,
           (hscalinteger*)incx
    );
}

/*------------------------------------------------------------------*/
