
#include <math.h>
#include <sys/time.h>
#include <stdio.h>
#include <stdlib.h>

#include "../../include/hscalapack.h"

/*------------------------------------------------------------------*/

int hscal_cscal_benchmark(
    int     *n, 
    hscalcomplex *ca, 
    hscalcomplex *cx, 
    int     *incx
)
{
    return hscal_cscal_benchmark_i(
           (hscalinteger*)n,
           ca,
           cx,
           (hscalinteger*)incx
    );
}

/*------------------------------------------------------------------*/

int hscal_cscal_benchmark_i(
    hscalinteger *n, 
    hscalcomplex *ca, 
    hscalcomplex *cx, 
    hscalinteger *incx
)
{
    /* System generated locals */
    hscalinteger i__1, i__2, i__3, i__4;
    hscalcomplex q__1;
    /* Local variables */
    static hscalinteger i__, nincx;
/*     scales a vector by a constant.   
       jack dongarra, linpack,  3/11/78.   
       modified 3/93 to return if incx .le. 0.   
       modified 12/3/93, array(1) declarations changed to array(*)   
       Parameter adjustments */
    --cx;
    /* Function Body */
    if (*n <= 0 || *incx <= 0) {
	return HSCAL_SUCCESS;
    }
    if (*incx == 1) {
	goto L20;
    }
/*        code for increment not equal to 1 */
    nincx = *n * *incx;
    i__1 = nincx;
    i__2 = *incx;
    for (i__ = 1; i__2 < 0 ? i__ >= i__1 : i__ <= i__1; i__ += i__2) {
	i__3 = i__;
	i__4 = i__;
	q__1.r = ca->r * cx[i__4].r - ca->i * cx[i__4].i, q__1.i = ca->r * cx[
		i__4].i + ca->i * cx[i__4].r;
	cx[i__3].r = q__1.r, cx[i__3].i = q__1.i;
/* L10: */
    }
    return HSCAL_SUCCESS;
/*        code for increment equal to 1 */
L20:
    i__2 = *n;
    for (i__ = 1; i__ <= i__2; ++i__) {
	i__1 = i__;
	i__3 = i__;
	q__1.r = ca->r * cx[i__3].r - ca->i * cx[i__3].i, q__1.i = ca->r * cx[
		i__3].i + ca->i * cx[i__3].r;
	cx[i__1].r = q__1.r, cx[i__1].i = q__1.i;
/* L30: */
    }
    return HSCAL_SUCCESS;
} /* cscal_ */

/*------------------------------------------------------------------*/

int hscal_dscal_benchmark(
    int     *n, 
    hscaldoublereal *da, 
    hscaldoublereal *dx, 
    int     *incx
)
{
    return hscal_dscal_benchmark_i(
           (hscalinteger*)n,
           da,
           dx,
           (hscalinteger*)incx
    );
}

/*------------------------------------------------------------------*/

int hscal_dscal_benchmark_i(
    hscalinteger *n, 
    hscaldoublereal *da, 
    hscaldoublereal *dx, 
    hscalinteger *incx
)
{
    /* System generated locals */
    hscalinteger i__1, i__2;
    /* Local variables */
    static hscalinteger i__, m, nincx, mp1;
/*     scales a vector by a constant.   
       uses unrolled loops for increment equal to one.   
       jack dongarra, linpack, 3/11/78.   
       modified 3/93 to return if incx .le. 0.   
       modified 12/3/93, array(1) declarations changed to array(*)   
       Parameter adjustments */
    --dx;
    /* Function Body */
    if (*n <= 0 || *incx <= 0) {
	return HSCAL_SUCCESS;
    }
    if (*incx == 1) {
	goto L20;
    }
/*        code for increment not equal to 1 */
    nincx = *n * *incx;
    i__1 = nincx;
    i__2 = *incx;
    for (i__ = 1; i__2 < 0 ? i__ >= i__1 : i__ <= i__1; i__ += i__2) {
	dx[i__] = *da * dx[i__];
/* L10: */
    }
    return HSCAL_SUCCESS;
/*        code for increment equal to 1   
          clean-up loop */
L20:
    m = *n % 5;
    if (m == 0) {
	goto L40;
    }
    i__2 = m;
    for (i__ = 1; i__ <= i__2; ++i__) {
	dx[i__] = *da * dx[i__];
/* L30: */
    }
    if (*n < 5) {
	return HSCAL_SUCCESS;
    }
L40:
    mp1 = m + 1;
    i__2 = *n;
    for (i__ = mp1; i__ <= i__2; i__ += 5) {
	dx[i__] = *da * dx[i__];
	dx[i__ + 1] = *da * dx[i__ + 1];
	dx[i__ + 2] = *da * dx[i__ + 2];
	dx[i__ + 3] = *da * dx[i__ + 3];
	dx[i__ + 4] = *da * dx[i__ + 4];
/* L50: */
    }
    return HSCAL_SUCCESS;
} /* dscal_ */

/*------------------------------------------------------------------*/

int hscal_sscal_benchmark(
    int  *n, 
    hscalreal *sa, 
    hscalreal *sx, 
    int  *incx
)
{
    return hscal_sscal_benchmark_i(
           (hscalinteger*)n,
           sa,
           sx,
           (hscalinteger*)incx
    );
}

/*------------------------------------------------------------------*/

int hscal_sscal_benchmark_i(
    hscalinteger *n, 
    hscalreal *sa, 
    hscalreal *sx, 
    hscalinteger *incx
)
{
    /* System generated locals */
    hscalinteger i__1, i__2;
    /* Local variables */
    static hscalinteger i__, m, nincx, mp1;
/*     scales a vector by a constant.   
       uses unrolled loops for increment equal to 1.   
       jack dongarra, linpack, 3/11/78.   
       modified 3/93 to return if incx .le. 0.   
       modified 12/3/93, array(1) declarations changed to array(*)   
       Parameter adjustments */
    --sx;
    /* Function Body */
    if (*n <= 0 || *incx <= 0) {
	return HSCAL_SUCCESS;
    }
    if (*incx == 1) {
	goto L20;
    }
/*        code for increment not equal to 1 */
    nincx = *n * *incx;
    i__1 = nincx;
    i__2 = *incx;
    for (i__ = 1; i__2 < 0 ? i__ >= i__1 : i__ <= i__1; i__ += i__2) {
	sx[i__] = *sa * sx[i__];
/* L10: */
    }
    return HSCAL_SUCCESS;
/*        code for increment equal to 1   
          clean-up loop */
L20:
    m = *n % 5;
    if (m == 0) {
	goto L40;
    }
    i__2 = m;
    for (i__ = 1; i__ <= i__2; ++i__) {
	sx[i__] = *sa * sx[i__];
/* L30: */
    }
    if (*n < 5) {
	return HSCAL_SUCCESS;
    }
L40:
    mp1 = m + 1;
    i__2 = *n;
    for (i__ = mp1; i__ <= i__2; i__ += 5) {
	sx[i__] = *sa * sx[i__];
	sx[i__ + 1] = *sa * sx[i__ + 1];
	sx[i__ + 2] = *sa * sx[i__ + 2];
	sx[i__ + 3] = *sa * sx[i__ + 3];
	sx[i__ + 4] = *sa * sx[i__ + 4];
/* L50: */
    }
    return HSCAL_SUCCESS;
} /* sscal_ */

/*------------------------------------------------------------------*/

int hscal_zscal_benchmark(
    int           *n, 
    hscaldoublecomplex *za, 
    hscaldoublecomplex *zx, 
    int           *incx
)
{
    return hscal_zscal_benchmark_i(
           (hscalinteger*)n,
           za,
           zx,
           (hscalinteger*)incx
    );
}

/*------------------------------------------------------------------*/

int hscal_zscal_benchmark_i(
    hscalinteger *n, 
    hscaldoublecomplex *za, 
    hscaldoublecomplex *zx, 
    hscalinteger *incx
)
{
    /* System generated locals */
    hscalinteger i__1, i__2, i__3;
    hscaldoublecomplex z__1;
    /* Local variables */
    static hscalinteger i__, ix;
/*     scales a vector by a constant.   
       jack dongarra, 3/11/78.   
       modified 3/93 to return if incx .le. 0.   
       modified 12/3/93, array(1) declarations changed to array(*)   
       Parameter adjustments */
    --zx;
    /* Function Body */
    if (*n <= 0 || *incx <= 0) {
	return HSCAL_SUCCESS;
    }
    if (*incx == 1) {
	goto L20;
    }
/*        code for increment not equal to 1 */
    ix = 1;
    i__1 = *n;
    for (i__ = 1; i__ <= i__1; ++i__) {
	i__2 = ix;
	i__3 = ix;
	z__1.r = za->r * zx[i__3].r - za->i * zx[i__3].i, z__1.i = za->r * zx[
		i__3].i + za->i * zx[i__3].r;
	zx[i__2].r = z__1.r, zx[i__2].i = z__1.i;
	ix += *incx;
/* L10: */
    }
    return HSCAL_SUCCESS;
/*        code for increment equal to 1 */
L20:
    i__1 = *n;
    for (i__ = 1; i__ <= i__1; ++i__) {
	i__2 = i__;
	i__3 = i__;
	z__1.r = za->r * zx[i__3].r - za->i * zx[i__3].i, z__1.i = za->r * zx[
		i__3].i + za->i * zx[i__3].r;
	zx[i__2].r = z__1.r, zx[i__2].i = z__1.i;
/* L30: */
    }
    return HSCAL_SUCCESS;
} /* zscal_ */

/*------------------------------------------------------------------*/

int hscal_csscal_benchmark(
    int           *n, 
    hscalreal          *sa, 
    hscalcomplex       *cx, 
    int           *incx
)
{
    return hscal_csscal_benchmark_i(
           (hscalinteger*)n,
           sa,
           cx,
           (hscalinteger*)incx
    );
}

/*------------------------------------------------------------------*/

int hscal_csscal_benchmark_i(
    hscalinteger *n, 
    hscalreal    *sa, 
    hscalcomplex *cx, 
    hscalinteger *incx
)
{
    /* System generated locals */
    hscalinteger i__1, i__2, i__3, i__4;
    hscalreal r__1, r__2;
    hscalcomplex q__1;
    /* Local variables */
    static hscalinteger i__, nincx;
/*     scales a hscalcomplex vector by a hscalreal constant.   
       jack dongarra, linpack, 3/11/78.   
       modified 3/93 to return if incx .le. 0.   
       modified 12/3/93, array(1) declarations changed to array(*)   
       Parameter adjustments */
    --cx;
    /* Function Body */
    if (*n <= 0 || *incx <= 0) {
	return HSCAL_SUCCESS;
    }
    if (*incx == 1) {
	goto L20;
    }
/*        code for increment not equal to 1 */
    nincx = *n * *incx;
    i__1 = nincx;
    i__2 = *incx;
    for (i__ = 1; i__2 < 0 ? i__ >= i__1 : i__ <= i__1; i__ += i__2) {
	i__3 = i__;
	i__4 = i__;
	r__1 = *sa * cx[i__4].r;
	r__2 = *sa * hscal_r_imag(&cx[i__]);
	q__1.r = r__1, q__1.i = r__2;
	cx[i__3].r = q__1.r, cx[i__3].i = q__1.i;
/* L10: */
    }
    return HSCAL_SUCCESS;
/*        code for increment equal to 1 */
L20:
    i__2 = *n;
    for (i__ = 1; i__ <= i__2; ++i__) {
	i__1 = i__;
	i__3 = i__;
	r__1 = *sa * cx[i__3].r;
	r__2 = *sa * hscal_r_imag(&cx[i__]);
	q__1.r = r__1, q__1.i = r__2;
	cx[i__1].r = q__1.r, cx[i__1].i = q__1.i;
/* L30: */
    }
    return HSCAL_SUCCESS;
} /* csscal_ */

/*------------------------------------------------------------------*/

int hscal_zdscal_benchmark(
    int           *n, 
    hscaldoublereal    *da, 
    hscaldoublecomplex *zx, 
    int           *incx
)
{
    return hscal_zdscal_benchmark_i(
           (hscalinteger*)n,
           da,
           zx,
           (hscalinteger*)incx
    );
}

/*------------------------------------------------------------------*/

int hscal_zdscal_benchmark_i(
    hscalinteger *n, 
    hscaldoublereal *da, 
    hscaldoublecomplex *zx, 
    hscalinteger *incx
)
{
    /* System generated locals */
    hscalinteger i__1, i__2, i__3;
    hscaldoublecomplex z__1, z__2;
    /* Local variables */
    static hscalinteger i__, ix;
/*     scales a vector by a constant.   
       jack dongarra, 3/11/78.   
       modified 3/93 to return if incx .le. 0.   
       modified 12/3/93, array(1) declarations changed to array(*)   
       Parameter adjustments */
    --zx;
    /* Function Body */
    if (*n <= 0 || *incx <= 0) {
	return HSCAL_SUCCESS;
    }
    if (*incx == 1) {
	goto L20;
    }
/*        code for increment not equal to 1 */
    ix = 1;
    i__1 = *n;
    for (i__ = 1; i__ <= i__1; ++i__) {
	i__2 = ix;
	z__2.r = *da, z__2.i = 0.;
	i__3 = ix;
	z__1.r = z__2.r * zx[i__3].r - z__2.i * zx[i__3].i, z__1.i = z__2.r * 
		zx[i__3].i + z__2.i * zx[i__3].r;
	zx[i__2].r = z__1.r, zx[i__2].i = z__1.i;
	ix += *incx;
/* L10: */
    }
    return HSCAL_SUCCESS;
/*        code for increment equal to 1 */
L20:
    i__1 = *n;
    for (i__ = 1; i__ <= i__1; ++i__) {
	i__2 = i__;
	z__2.r = *da, z__2.i = 0.;
	i__3 = i__;
	z__1.r = z__2.r * zx[i__3].r - z__2.i * zx[i__3].i, z__1.i = z__2.r * 
		zx[i__3].i + z__2.i * zx[i__3].r;
	zx[i__2].r = z__1.r, zx[i__2].i = z__1.i;
/* L30: */
    }
    return HSCAL_SUCCESS;
} /* zdscal_ */


/*------------------------------------------------------------------*/

