
#include <math.h>
#include <sys/time.h>
#include <stdio.h>
#include <stdlib.h>

#include "../../include/hscalapack.h"

/*------------------------------------------------------------------*/

int hscal_cswap_benchmark(
    int     *n, 
    hscalcomplex *cx, 
    int     *incx, 
    hscalcomplex *cy, 
    int     *incy
)
{
    return hscal_cswap_benchmark_i(
           (hscalinteger*)n,
           cx,
           (hscalinteger*)incx,
           cy,
           (hscalinteger*)incy
    );
}

/*------------------------------------------------------------------*/

int hscal_cswap_benchmark_i(
    hscalinteger *n, 
    hscalcomplex *cx, 
    hscalinteger *incx, 
    hscalcomplex *cy, 
    hscalinteger *incy
)
{
    /* System generated locals */
    hscalinteger i__1, i__2, i__3;
    /* Local variables */
    static hscalinteger i__;
    static hscalcomplex ctemp;
    static hscalinteger ix, iy;
/*     interchanges two vectors.   
       jack dongarra, linpack, 3/11/78.   
       modified 12/3/93, array(1) declarations changed to array(*)   
       Parameter adjustments */
    --cy;
    --cx;
    /* Function Body */
    if (*n <= 0) {
	return HSCAL_SUCCESS;
    }
    if (*incx == 1 && *incy == 1) {
	goto L20;
    }
/*       code for unequal increments or equal increments not equal   
           to 1 */
    ix = 1;
    iy = 1;
    if (*incx < 0) {
	ix = (-(*n) + 1) * *incx + 1;
    }
    if (*incy < 0) {
	iy = (-(*n) + 1) * *incy + 1;
    }
    i__1 = *n;
    for (i__ = 1; i__ <= i__1; ++i__) {
	i__2 = ix;
	ctemp.r = cx[i__2].r, ctemp.i = cx[i__2].i;
	i__2 = ix;
	i__3 = iy;
	cx[i__2].r = cy[i__3].r, cx[i__2].i = cy[i__3].i;
	i__2 = iy;
	cy[i__2].r = ctemp.r, cy[i__2].i = ctemp.i;
	ix += *incx;
	iy += *incy;
/* L10: */
    }
    return HSCAL_SUCCESS;
/*       code for both increments equal to 1 */
L20:
    i__1 = *n;
    for (i__ = 1; i__ <= i__1; ++i__) {
	i__2 = i__;
	ctemp.r = cx[i__2].r, ctemp.i = cx[i__2].i;
	i__2 = i__;
	i__3 = i__;
	cx[i__2].r = cy[i__3].r, cx[i__2].i = cy[i__3].i;
	i__2 = i__;
	cy[i__2].r = ctemp.r, cy[i__2].i = ctemp.i;
/* L30: */
    }
    return HSCAL_SUCCESS;
} /* cswap_ */

/*------------------------------------------------------------------*/

int hscal_dswap_benchmark(
    int     *n, 
    hscaldoublereal *dx, 
    int     *incx, 
    hscaldoublereal *dy, 
    int     *incy
)
{
    return hscal_dswap_benchmark_i(
           (hscalinteger*)n,
           dx,
           (hscalinteger*)incx,
           dy,
           (hscalinteger*)incy
    );
}

/*------------------------------------------------------------------*/

int hscal_dswap_benchmark_i(
    hscalinteger *n, 
    hscaldoublereal *dx, 
    hscalinteger *incx, 
    hscaldoublereal *dy, 
    hscalinteger *incy
)
{
    /* System generated locals */
    hscalinteger i__1;
    /* Local variables */
    static hscalinteger i__, m;
    static hscaldoublereal dtemp;
    static hscalinteger ix, iy, mp1;
/*     interchanges two vectors.   
       uses unrolled loops for increments equal one.   
       jack dongarra, linpack, 3/11/78.   
       modified 12/3/93, array(1) declarations changed to array(*)   
       Parameter adjustments */
    --dy;
    --dx;
    /* Function Body */
    if (*n <= 0) {
	return HSCAL_SUCCESS;
    }
    if (*incx == 1 && *incy == 1) {
	goto L20;
    }
/*       code for unequal increments or equal increments not equal   
           to 1 */
    ix = 1;
    iy = 1;
    if (*incx < 0) {
	ix = (-(*n) + 1) * *incx + 1;
    }
    if (*incy < 0) {
	iy = (-(*n) + 1) * *incy + 1;
    }
    i__1 = *n;
    for (i__ = 1; i__ <= i__1; ++i__) {
	dtemp = dx[ix];
	dx[ix] = dy[iy];
	dy[iy] = dtemp;
	ix += *incx;
	iy += *incy;
/* L10: */
    }
    return HSCAL_SUCCESS;
/*       code for both increments equal to 1   
         clean-up loop */
L20:
    m = *n % 3;
    if (m == 0) {
	goto L40;
    }
    i__1 = m;
    for (i__ = 1; i__ <= i__1; ++i__) {
	dtemp = dx[i__];
	dx[i__] = dy[i__];
	dy[i__] = dtemp;
/* L30: */
    }
    if (*n < 3) {
	return HSCAL_SUCCESS;
    }
L40:
    mp1 = m + 1;
    i__1 = *n;
    for (i__ = mp1; i__ <= i__1; i__ += 3) {
	dtemp = dx[i__];
	dx[i__] = dy[i__];
	dy[i__] = dtemp;
	dtemp = dx[i__ + 1];
	dx[i__ + 1] = dy[i__ + 1];
	dy[i__ + 1] = dtemp;
	dtemp = dx[i__ + 2];
	dx[i__ + 2] = dy[i__ + 2];
	dy[i__ + 2] = dtemp;
/* L50: */
    }
    return HSCAL_SUCCESS;
} /* dswap_ */

/*------------------------------------------------------------------*/

int hscal_sswap_benchmark(
    int     *n, 
    hscalreal *sx, 
    int     *incx,
    hscalreal *sy, 
    int     *incy
)
{
    return hscal_sswap_benchmark_i(
           (hscalinteger*)n,
           sx,
           (hscalinteger*)incx,
           sy,
           (hscalinteger*)incy
    );
}

/*------------------------------------------------------------------*/

int hscal_sswap_benchmark_i(
    hscalinteger *n, 
    hscalreal *sx, 
    hscalinteger *incx,
    hscalreal *sy, 
    hscalinteger *incy
)
{
    /* System generated locals */
    hscalinteger i__1;
    /* Local variables */
    static hscalinteger i__, m;
    static hscalreal stemp;
    static hscalinteger ix, iy, mp1;
/*     interchanges two vectors.   
       uses unrolled loops for increments equal to 1.   
       jack dongarra, linpack, 3/11/78.   
       modified 12/3/93, array(1) declarations changed to array(*)   
       Parameter adjustments */
    --sy;
    --sx;
    /* Function Body */
    if (*n <= 0) {
	return HSCAL_SUCCESS;
    }
    if (*incx == 1 && *incy == 1) {
	goto L20;
    }
/*       code for unequal increments or equal increments not equal   
           to 1 */
    ix = 1;
    iy = 1;
    if (*incx < 0) {
	ix = (-(*n) + 1) * *incx + 1;
    }
    if (*incy < 0) {
	iy = (-(*n) + 1) * *incy + 1;
    }
    i__1 = *n;
    for (i__ = 1; i__ <= i__1; ++i__) {
	stemp = sx[ix];
	sx[ix] = sy[iy];
	sy[iy] = stemp;
	ix += *incx;
	iy += *incy;
/* L10: */
    }
    return HSCAL_SUCCESS;
/*       code for both increments equal to 1   
         clean-up loop */
L20:
    m = *n % 3;
    if (m == 0) {
	goto L40;
    }
    i__1 = m;
    for (i__ = 1; i__ <= i__1; ++i__) {
	stemp = sx[i__];
	sx[i__] = sy[i__];
	sy[i__] = stemp;
/* L30: */
    }
    if (*n < 3) {
	return HSCAL_SUCCESS;
    }
L40:
    mp1 = m + 1;
    i__1 = *n;
    for (i__ = mp1; i__ <= i__1; i__ += 3) {
	stemp = sx[i__];
	sx[i__] = sy[i__];
	sy[i__] = stemp;
	stemp = sx[i__ + 1];
	sx[i__ + 1] = sy[i__ + 1];
	sy[i__ + 1] = stemp;
	stemp = sx[i__ + 2];
	sx[i__ + 2] = sy[i__ + 2];
	sy[i__ + 2] = stemp;
/* L50: */
    }
    return HSCAL_SUCCESS;
} /* sswap_ */

/*------------------------------------------------------------------*/

int hscal_zswap_benchmark(
    int     *n, 
    hscaldoublecomplex *zx, 
    int     *incx, 
    hscaldoublecomplex *zy, 
    int     *incy
)
{
    return hscal_zswap_benchmark_i(
           (hscalinteger*)n,
           zx,
           (hscalinteger*)incx,
           zy,
           (hscalinteger*)incy
    );
}

/*------------------------------------------------------------------*/

int hscal_zswap_benchmark_i(
    hscalinteger *n, 
    hscaldoublecomplex *zx, 
    hscalinteger *incx, 
    hscaldoublecomplex *zy, 
    hscalinteger *incy
)
{
    /* System generated locals */
    hscalinteger i__1, i__2, i__3;
    /* Local variables */
    static hscalinteger i__;
    static hscaldoublecomplex ztemp;
    static hscalinteger ix, iy;
/*     interchanges two vectors.   
       jack dongarra, 3/11/78.   
       modified 12/3/93, array(1) declarations changed to array(*)   
       Parameter adjustments */
    --zy;
    --zx;
    /* Function Body */
    if (*n <= 0) {
	return HSCAL_SUCCESS;
    }
    if (*incx == 1 && *incy == 1) {
	goto L20;
    }
/*       code for unequal increments or equal increments not equal   
           to 1 */
    ix = 1;
    iy = 1;
    if (*incx < 0) {
	ix = (-(*n) + 1) * *incx + 1;
    }
    if (*incy < 0) {
	iy = (-(*n) + 1) * *incy + 1;
    }
    i__1 = *n;
    for (i__ = 1; i__ <= i__1; ++i__) {
	i__2 = ix;
	ztemp.r = zx[i__2].r, ztemp.i = zx[i__2].i;
	i__2 = ix;
	i__3 = iy;
	zx[i__2].r = zy[i__3].r, zx[i__2].i = zy[i__3].i;
	i__2 = iy;
	zy[i__2].r = ztemp.r, zy[i__2].i = ztemp.i;
	ix += *incx;
	iy += *incy;
/* L10: */
    }
    return HSCAL_SUCCESS;
/*       code for both increments equal to 1 */
L20:
    i__1 = *n;
    for (i__ = 1; i__ <= i__1; ++i__) {
	i__2 = i__;
	ztemp.r = zx[i__2].r, ztemp.i = zx[i__2].i;
	i__2 = i__;
	i__3 = i__;
	zx[i__2].r = zy[i__3].r, zx[i__2].i = zy[i__3].i;
	i__2 = i__;
	zy[i__2].r = ztemp.r, zy[i__2].i = ztemp.i;
/* L30: */
    }
    return HSCAL_SUCCESS;
} /* zswap_ */

/*------------------------------------------------------------------*/
