
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>

#include <hmpi.h>
#include <f2c.h>
#include <scalapack.h>

#include "problem_parameters.h"

int main(int argc, char **argv)
{
    int     DESCA[DLEN_];
    int     i, rc;
    int     ictxt, myrow, mycol;
    int     Ka;
    int     p = nprow;
    int     q = npcol;

    int     Ai, Aj, Ad[DLEN_], Ad0[DLEN_];
    int     Aii, Ajj, Ald, Aimb1, Ainb1, Amb, Anb, Arow, Acol;

    MPI_Comm testcomm;

    //
    // Initialize HMPI runtime
    HMPI_Init(&argc, &argv);

    testcomm = *(MPI_Comm*)HMPI_Get_comm(HMPI_COMM_WORLD_GROUP);

    /*
     * Translate algocomm to a BLACS handle
     */
    ictxt = Csys2blacs_handle(testcomm);

    /*
     * Form BLACS context based on algocomm
     */
    Cblacs_gridinit(&ictxt, "r", p, q);
    blacs_gridinfo__(&ictxt, &p, &q, &myrow, &mycol);

    DESCA[DTYPE_] = BLOCK_CYCLIC_2D_INB;
    DESCA[CTXT_]  = ictxt;
    DESCA[M_]     = M;
    DESCA[N_]     = N;
    DESCA[IMB_]   = MB;
    DESCA[INB_]   = NB;
    DESCA[MB_]    = MB;
    DESCA[NB_]    = NB;
    DESCA[RSRC_]  = 0;
    DESCA[CSRC_]  = 0;

    DESCA[LLD_]   = max(1, PB_Cnumroc(DESCA[M_],
                                      1,
                                      DESCA[IMB_],
                                      DESCA[MB_],
                                      myrow,
                                      DESCA[RSRC_],
                                      p
                       )
    );

    Ka            = max(1, PB_Cnumroc(JA+DESCA[N_]-1,
                                      1,
                                      DESCA[INB_],
                                      DESCA[NB_],
                                      mycol,
                                      DESCA[CSRC_],
                                      q
                       )
    );

    PB_CargFtoC( IA, JA, DESCA, &Ai, &Aj, Ad );

    PB_Cinfog2l( Ai, Aj, Ad, nprow, npcol, myrow, mycol,
                 &Aii, &Ajj, &Arow, &Acol );
   
    printf( 
       "(%d, %d)==> Aii=%d, Ajj=%d, Arow=%d, Acol=%d\n",
       myrow, mycol, Aii, Ajj, Arow, Acol
    );

    blacs_gridexit__(&ictxt);

    HMPI_Finalize(0);
}

