
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>

#include <hscalapack.h>

#include "hscalapack_pzgemm_bsize_parameters.h"

/*------------------------------------------------------------------*/

double hscal_pzgemm_benchmark_function_t
(
     const void* input_p,
     int num_of_p,
     void* output_p
)
{
   int i, *params = (int*)input_p;
   double btime;

   int TRANSAI    =  params[0];
   int TRANSBI    =  params[1];
   int M          =  params[1+1];
   int N          =  params[1+1+1];
   int K          =  params[1+1+1+1];
   int IA         =  params[1+1+1+1+CNARG+1];
   int JA         =  params[1+1+1+1+CNARG+1+1];
   int *DESCA     = &params[1+1+1+1+CNARG+1+1+1];
   int IB         =  params[1+1+1+1+CNARG+1+1+1+DLEN_];
   int JB         =  params[1+1+1+1+CNARG+1+1+1+DLEN_+1];
   int *DESCB     = &params[1+1+1+1+CNARG+1+1+1+DLEN_+1+1];
   int IC         =  params[1+1+1+1+CNARG+1+1+1+DLEN_+1+1+DLEN_+CNARG];
   int JC         =  params[1+1+1+1+CNARG+1+1+1+DLEN_+1+1+DLEN_+CNARG+1];
   int *DESCC     = &params[1+1+1+1+CNARG+1+1+1+DLEN_+1+1+DLEN_+CNARG+1+1];

   char TRANSAC   = TRANSAI;
   char TRANSA[2] = { TRANSAC, '\0' };

   char TRANSBC   = TRANSBI;
   char TRANSB[2] = { TRANSBC, '\0' };

   hscaldoublecomplex ALPHA, BETA;
   hscaldoublecomplex *a, *b, *c;

   int reconm     = params[1+1+1+1+1+1+1+1+1+DLEN_+1+1+DLEN_+1+1+1+1+DLEN_];
   int reconn     = params[1+1+1+1+1+1+1+1+1+DLEN_+1+1+DLEN_+1+1+1+1+DLEN_+1];
   int reconk     = params[1+1+1+1+1+1+1+1+1+DLEN_+1+1+DLEN_+1+1+1+1+DLEN_+1+1];
    
   int lda, ldb, ldc;

   ALPHA.r        = params[1+1+1+1+1];
   ALPHA.i        = params[1+1+1+1+1+1];

   BETA.r         = params[1+1+1+1+CNARG+1+1+1+DLEN_+1+1+DLEN_];
   BETA.i         = params[1+1+1+1+CNARG+1+1+1+DLEN_+1+1+DLEN_+1];

   if (TRANSAC == 'N')
   {
      lda = reconm;
   }
   else
   {
      lda = reconk;
   }

   if (TRANSBC == 'N')
   {
      ldb = reconk;
   }
   else
   {
      ldb = reconn;
   }

   ldc = reconm;

   c = (hscaldoublecomplex*)malloc(
                 sizeof(hscaldoublecomplex)
                 *
                 (reconm*reconn)
   );

   if (c == NULL)
   {
      *(int*)output_p = HSCAL_ERR_LOW_MEMORY;
      return HSCAL_ERR_RECON_ESTIMATION;
   }

   for (i = 0; i < (reconm*reconn); i++)
   {
      c[i].r = 1.;
      c[i].i = 1.;
   }

   btime = MPI_Wtime();

   //
   // Allocate the matrices A, B, and C
   a = (hscaldoublecomplex*)malloc(
                 sizeof(hscaldoublecomplex)
                 *
                 (reconm*reconk)
   );

   if (a == NULL)
   {
      *(int*)output_p = HSCAL_ERR_LOW_MEMORY;
      return HSCAL_ERR_RECON_ESTIMATION;
   }

   b = (hscaldoublecomplex*)malloc(
                 sizeof(hscaldoublecomplex)
                 *
                 (reconk*reconn)
   );

   if (b == NULL)
   {
      *(int*)output_p = HSCAL_ERR_LOW_MEMORY;
      return HSCAL_ERR_RECON_ESTIMATION;
   }

   //
   // Initialize the matrices A, B, and C
   for (i = 0; i < (reconm*reconk); i++)
   {
      a[i].r = 2.;
      a[i].i = 2.;
   }

   for (i = 0; i < (reconk*reconn); i++)
   {
      b[i].r = 2.;
      b[i].i = 2.;
   }

   //
   // Call the BLAS zgemm routine
   hscal_zgemm_benchmark(
        TRANSA,
        TRANSB,
        &reconm,
        &reconn,
        &reconk,
        &ALPHA,
        a,
        &lda,
        b,
        &ldb,
        &BETA,
        c,
        &ldc
   );

   //
   // Free the matrices A, B, and C
   free(a);
   free(b);

   btime = MPI_Wtime() - btime;

   free(c);

   *(int*)output_p = HSCAL_SUCCESS;

   return btime;
}

/*------------------------------------------------------------------*/

int hscal_pack_pzgemm_modelp_noauto_t(
   int * reconm,
   int * reconn,
   int * reconk,
   F_CHAR_T TRANSA, 
   F_CHAR_T TRANSB,
   int * M,
   int * N, 
   int * K,
   double * ALPHA,
   int * IA, 
   int * JA, 
   int * DESCA,
   int * IB, 
   int * JB, 
   int * DESCB,
   double * BETA,
   int * IC, 
   int * JC, 
   int * DESCC,
   int **model_params,
   int *model_count
)
{
   int i;

   /*
   * Just get a indicative multiplication scalar of 
   * alpha and beta
   */
   int alphai[CNARG], betai[CNARG];

   if (ALPHA[REAL_PART] == 0.)
   {
      alphai[REAL_PART] = 0;
   }
   else if (ALPHA[REAL_PART] == 1.)
   {
      alphai[REAL_PART] = 1;
   }
   else
   {
      alphai[REAL_PART] = 2;
   }

   if (ALPHA[IMAG_PART] == 0.)
   {
      alphai[IMAG_PART] = 0;
   }
   else if (ALPHA[IMAG_PART] == 1.)
   {
      alphai[IMAG_PART] = 1;
   }
   else
   {
      alphai[IMAG_PART] = 2;
   }

   if (BETA[REAL_PART] == 0.)
   {
      betai[REAL_PART] = 0;
   }
   else if (BETA[REAL_PART] == 1.)
   {
      betai[REAL_PART] = 1;
   }
   else
   {
      betai[REAL_PART] = 2;
   }

   if (BETA[IMAG_PART] == 0.)
   {
      betai[IMAG_PART] = 0;
   }
   else if (BETA[IMAG_PART] == 1.)
   {
      betai[IMAG_PART] = 1;
   }
   else
   {
      betai[IMAG_PART] = 2;
   }

   *model_count = 1        /* TRANSA */
                  +        
                  1        /* TRANSB */
                  +        
                  1        /* M */
                  +        
                  1        /* N */
                  +        
                  1        /* K */
                  +
                  CNARG    /* ALPHA */
                  +
                  1        /* IA */
                  +
                  1        /* JA */ 
                  +
                  DLEN_    /* DESCA */
                  +
                  1        /* IB */
                  +
                  1        /* JB */ 
                  +
                  DLEN_    /* DESCB */
                  +
                  CNARG    /* BETA */
                  +
                  1        /* IC */
                  +
                  1        /* JC */
                  +
                  DLEN_    /* DESCC */
                  +
                  1        /* reconm */
                  +
                  1        /* reconn */
                  +
                  1        /* reconk */
   ;

   *model_params = (int*)calloc(
                         (*model_count),
                         sizeof(int)
   );

   if (*model_params == NULL)
   {
      return MPC_ERR_NOMEM;
   }

   (*model_params)[0]                                                 = *TRANSA;
   (*model_params)[1]                                                 = *TRANSB;
   (*model_params)[1+1]                                               = *M;
   (*model_params)[1+1+1]                                             = *N;
   (*model_params)[1+1+1+1]                                           = *K;
   (*model_params)[1+1+1+1+1]                                         = alphai[REAL_PART];
   (*model_params)[1+1+1+1+1+1]                                       = alphai[IMAG_PART];
   (*model_params)[1+1+1+1+1+1+1]                                     = *IA;
   (*model_params)[1+1+1+1+1+1+1+1]                                   = *JA;
   (*model_params)[1+1+1+1+1+1+1+1+1+DLEN_]                           = *IB;
   (*model_params)[1+1+1+1+1+1+1+1+1+DLEN_+1]                         = *JB;
   (*model_params)[1+1+1+1+1+1+1+1+1+DLEN_+1+1+DLEN_]                 = betai[REAL_PART];
   (*model_params)[1+1+1+1+1+1+1+1+1+DLEN_+1+1+DLEN_+1]               = betai[IMAG_PART];
   (*model_params)[1+1+1+1+1+1+1+1+1+DLEN_+1+1+DLEN_+1+1]             = *IC;
   (*model_params)[1+1+1+1+1+1+1+1+1+DLEN_+1+1+DLEN_+1+1+1]           = *JC;
   (*model_params)[1+1+1+1+1+1+1+1+1+DLEN_+1+1+DLEN_+1+1+1+1+DLEN_]   = *reconm;
   (*model_params)[1+1+1+1+1+1+1+1+1+DLEN_+1+1+DLEN_+1+1+1+1+DLEN_+1] = *reconn;
   (*model_params)[1+1+1+1+1+1+1+1+1+DLEN_+1+1+DLEN_+1+1+1+1+DLEN_+1+1] = *reconk;

   for (i = 0; i < DLEN_; i++)
   {
       (*model_params)[1+1+1+1+1+1+1+1+1+i]                           = DESCA[i];
       (*model_params)[1+1+1+1+1+1+1+1+1+DLEN_+1+1+i]                 = DESCB[i];
       (*model_params)[1+1+1+1+1+1+1+1+1+DLEN_+1+1+DLEN_+1+1+1+1+i]   = DESCC[i];
   }

   return HSCAL_SUCCESS;
}

/*------------------------------------------------------------------*/

int hscal_pzgemm_recon_t(
   int * reconm,
   int * reconn,
   int * reconk,
   F_CHAR_T TRANSA, 
   F_CHAR_T TRANSB,
   int * M,
   int * N, 
   int * K,
   double * ALPHA,
   int * IA, 
   int * JA, 
   int * DESCA,
   int * IB, 
   int * JB, 
   int * DESCB,
   double * BETA,
   int * IC, 
   int * JC, 
   int * DESCC
)
{
   int rc;

   if (HMPI_Is_member(HMPI_COMM_WORLD_GROUP))
   {
      int output_p;
      int *model_params, model_count;

      rc = hscal_pack_pzgemm_modelp_noauto_t(
                reconm,
                reconn,
                reconk,
                TRANSA,
                TRANSB,
                M,
                N,
                K,
                ALPHA,
                IA,
                JA,
                DESCA,
                IB,
                JB,
                DESCB,
                BETA,
                IC,
                JC,
                DESCC,
                &model_params,
                &model_count  
      );      

      if (rc != HSCAL_SUCCESS)
      {
         return rc;
      }

      rc = HMPI_Recon_t(
               &hscal_pzgemm_benchmark_function_t,
               model_params,
               model_count,
               &output_p
      );

      if (rc != HMPI_SUCCESS)
      {
         free(model_params);
         return rc;
      }

      if (HMPI_Is_member(HMPI_PROC_WORLD_GROUP))
      {
         if (output_p != HSCAL_SUCCESS)
         {
            free(model_params);
            return output_p;
         }
      }

      free(model_params);        
   }

   return HSCAL_SUCCESS;
}

/*---------------------------------------------------------------------------*/

int main(int argc, char **argv)
{
    int        DESCA[DLEN_], DESCB[DLEN_], DESCC[DLEN_];
    int        i, rc;
    double     time;

    char TRANSAC = TRANSAI;
    char *TRANSA = &TRANSAC;
    char TRANSBC = TRANSBI;
    char *TRANSB = &TRANSBC;

    double algotime;

    //
    // Initialize HMPI runtime
    HMPI_Init(&argc, &argv);

    int num_of_computers = HMPI_Get_number_of_computers();
    int p                = sqrt(num_of_computers);
    int q                = num_of_computers/p;
    int reconm           = M/p;
    int reconn           = N/q;
    int min_m_n          = (reconm < reconn) ? reconm : reconn;
    int reconk;

    for (reconk = 16; reconk <= 512; reconk+=16)
    {
        if (TRANSAC == 'N')
        {
          DESCA[DTYPE_] = BLOCK_CYCLIC_2D_INB;
          DESCA[M_]     = M;
          DESCA[N_]     = K;
          DESCA[IMB_]   = reconk;
          DESCA[INB_]   = reconk;
          DESCA[MB_]    = reconk;
          DESCA[NB_]    = reconk;
          DESCA[RSRC_]  = 0;
          DESCA[CSRC_]  = 0;
        }
        else
        {
          DESCA[DTYPE_] = BLOCK_CYCLIC_2D_INB;
          DESCA[M_]     = K;
          DESCA[N_]     = M;
          DESCA[IMB_]   = reconk;
          DESCA[INB_]   = reconk;
          DESCA[MB_]    = reconk;
          DESCA[NB_]    = reconk;
          DESCA[RSRC_]  = 0;
          DESCA[CSRC_]  = 0;
        }
 
        if (TRANSBC == 'N')
        {
          DESCB[DTYPE_] = BLOCK_CYCLIC_2D_INB;
          DESCB[M_]     = K;
          DESCB[N_]     = N;
          DESCB[IMB_]   = reconk;
          DESCB[INB_]   = reconk;
          DESCB[MB_]    = reconk;
          DESCB[NB_]    = reconk;
          DESCB[RSRC_]  = 0;
          DESCB[CSRC_]  = 0;
        }
        else
        {
          DESCB[DTYPE_] = BLOCK_CYCLIC_2D_INB;
          DESCB[M_]     = N;
          DESCB[N_]     = K;
          DESCB[IMB_]   = reconk;
          DESCB[INB_]   = reconk;
          DESCB[MB_]    = reconk;
          DESCB[NB_]    = reconk;
          DESCB[RSRC_]  = 0;
          DESCB[CSRC_]  = 0;
        }

        DESCC[DTYPE_] = BLOCK_CYCLIC_2D_INB;
        DESCC[M_]     = M;
        DESCC[N_]     = N;
        DESCC[IMB_]   = reconk;
        DESCC[INB_]   = reconk;
        DESCC[MB_]    = reconk;
        DESCC[NB_]    = reconk;
        DESCC[RSRC_]  = 0;
        DESCC[CSRC_]  = 0;
 
        rc = hscal_pzgemm_recon_t(
                  &reconm,
                  &reconn,
                  &reconk,
                  TRANSA,
                  TRANSB,
                  &M,
                  &N,
                  &K,
                  ALPHA,
                  &IA,
                  &JA,
                  DESCA,
                  &IB,
                  &JB,
                  DESCB,
                  BETA,
                  &IC,
                  &JC,
                  DESCC
        );

        if (rc != HMPI_SUCCESS)
        {    
           return rc;
        }

        HMPI_Barrier(HMPI_COMM_WORLD_GROUP);

        if (HMPI_Is_host())
        {
            int *model_params, model_count;

            rc = hscal_pack_pzgemm_modelp_noauto_optimal_b(
                      TRANSA,
                      TRANSB,
                      &M,
                      &N,
                      &K,
                      ALPHA,
                      &IA,
                      &JA,
                      DESCA,
                      &IB,
                      &JB,
                      DESCB,
                      BETA,
                      &IC,
                      &JC,
                      DESCC,
                      &reconm,
                      &reconn,
                      &reconk,   /*  BLOCK SIZE  */
                      &p,
                      &q,
                      &model_params,
                      &model_count
            );

            if (rc != HSCAL_SUCCESS)
            {
               return rc;
            }

            //hscal_set_debug(HSCAL_LOG_VERBOSE);

            time = HMPI_Timeof(
                       hscal_model_pzgemm_optimal_b,
                       model_params,
                       model_count
            );

            printf(
               "HSCALAPACK =====> ZGEMM UPDATE: p=%d, q=%d, (%d,%d) x (%d,%d)=%f,"
               " Block size=%d\n",
               p,
               q,
               reconm,
               reconk,
               reconk,
               reconn,
               time,
               reconk
            );

            free(model_params);
        }
    }

    HMPI_Finalize(0);
}
