
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>

#include <hscalapack.h>

#include "hscalapack_pdgemm_parameters.h"

/*---------------------------------------------------------------------------*/

int main(int argc, char **argv)
{
    int        DESCA[DLEN_], DESCB[DLEN_], DESCC[DLEN_];
    int        i, rc;
    double     time;
    HMPI_Group gid;

    char TRANSAC = TRANSAI;
    char *TRANSA = &TRANSAC;
    char TRANSBC = TRANSBI;
    char *TRANSB = &TRANSBC;

    struct timeval gstart, gend;

    //
    // Initialize HMPI runtime
    HMPI_Init(&argc, &argv);

    if (TRANSAC == 'N')
    {
       DESCA[DTYPE_] = BLOCK_CYCLIC_2D_INB;
       DESCA[M_]     = M;
       DESCA[N_]     = K;
       DESCA[IMB_]   = MB;
       DESCA[INB_]   = KB;
       DESCA[MB_]    = MB;
       DESCA[NB_]    = KB;
       DESCA[RSRC_]  = 0;
       DESCA[CSRC_]  = 0;
    }
    else
    {
       DESCA[DTYPE_] = BLOCK_CYCLIC_2D_INB;
       DESCA[M_]     = K;
       DESCA[N_]     = M;
       DESCA[IMB_]   = KB;
       DESCA[INB_]   = MB;
       DESCA[MB_]    = KB;
       DESCA[NB_]    = MB;
       DESCA[RSRC_]  = 0;
       DESCA[CSRC_]  = 0;
    }

    if (TRANSBC == 'N')
    {
       DESCB[DTYPE_] = BLOCK_CYCLIC_2D_INB;
       DESCB[M_]     = K;
       DESCB[N_]     = N;
       DESCB[IMB_]   = KB;
       DESCB[INB_]   = NB;
       DESCB[MB_]    = KB;
       DESCB[NB_]    = NB;
       DESCB[RSRC_]  = 0;
       DESCB[CSRC_]  = 0;
    }
    else
    {
       DESCB[DTYPE_] = BLOCK_CYCLIC_2D_INB;
       DESCB[M_]     = N;
       DESCB[N_]     = K;
       DESCB[IMB_]   = NB;
       DESCB[INB_]   = KB;
       DESCB[MB_]    = NB;
       DESCB[NB_]    = KB;
       DESCB[RSRC_]  = 0;
       DESCB[CSRC_]  = 0;
    }

    DESCC[DTYPE_] = BLOCK_CYCLIC_2D_INB;
    DESCC[M_]     = M;
    DESCC[N_]     = N;
    DESCC[IMB_]   = MB;
    DESCC[INB_]   = NB;
    DESCC[MB_]    = MB;
    DESCC[NB_]    = NB;
    DESCC[RSRC_]  = 0;
    DESCC[CSRC_]  = 0;

    rc = hscal_pdgemm_recon(
              TRANSA,
              TRANSB,
              &M,
              &N,
              &K,
              &ALPHA,
              &IA,
              &JA,
              DESCA,
              &IB,
              &JB,
              DESCB,
              &BETA,
              &IC,
              &JC,
              DESCC
    );

    if (rc != HMPI_SUCCESS)
    {
       return rc;
    }

    {
       int *model_params, model_count;

       if (HMPI_Is_host())
       {
          gettimeofday(&gstart, NULL);
       }

       HMPI_Debug(1);

       rc = hscal_pack_pdgemm_modelp(
                 TRANSA,
                 TRANSB,
                 &M,
                 &N,
                 &K,
                 &ALPHA,
                 &IA,
                 &JA,
                 DESCA,
                 &IB,
                 &JB,
                 DESCB,
                 &BETA,
                 &IC,
                 &JC,
                 DESCC,
                 &model_params,
                 &model_count
       );

       if (rc != HSCAL_SUCCESS)
       {
          return rc;
       }

       HMPI_Debug(1);

       rc = HMPI_Group_heuristic_pauto_create(
                &gid,
                hscal_model_pdgemm,
                hscal_heuristic_fn_all,
                model_params,
                model_count
       );

       if (rc != HMPI_SUCCESS)
       {
          free(model_params);
          return rc;
       }

       free(model_params);

       if (HMPI_Is_host())
       {
          gettimeofday(&gend, NULL);
       }

       if (HMPI_Is_member(&gid))
       {
          HMPI_Group_free(
              &gid
          );
       }

       HMPI_Debug(0);
    }

    /*
    * Print Execution time
    */
    if (HMPI_Is_host())
    {
       double gtstart = gstart.tv_sec + gstart.tv_usec/1000000.;
       double gtend = gend.tv_sec + gend.tv_usec/1000000.;
       double gt_sec = gtend - gtstart;

       printf(
          "TRANSA=%c, TRANSB=%c, M=%d, N=%d, K=%d, MB=%d, NB=%d, KB=%d, "
          "Group creation time(sec)=%f\n",
          TRANSAC,
          TRANSBC,
          M,
          N,
          K,
          MB,
          NB,
          KB,
          gt_sec
       );
    }

    HMPI_Finalize(0);
}
